-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    20:39:36 08/11/05
-- Design Name:    utils.vhd - miscelaneous basic elements
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    standard elements to build registers and others
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

package UTILS is

  -- *****************************************************************
  -- FDCE : Basic flip flop component type D with Load enable and reset
  component cl_fdce
     port ( Q        : OUT std_logic;                                    -- flip flop output bit
            D        : IN  std_logic;                                    -- bit to load into flip flop when CE will be enabled
            C        : IN  std_logic;                                    -- clock
            CE       : IN  std_logic;                                    -- chip enable - allow D value to be loaded synchronously with CLK
            CLR      : IN  std_logic                                     -- clear internal value to zero
           );
  end component;

  -- *****************************************************************
  -- scREG : Generic register
  component cl_reg
     generic (  WIDTH   : INTEGER := 32 );                               -- width of the register
     port ( D        : IN  std_logic_vector( (WIDTH-1) downto 0 );       -- input data may stored next cycle
            Q        : OUT std_logic_vector( (WIDTH-1) downto 0 );       -- output value of the register
            CLK      : IN  std_logic;                                    -- system clock
            LOAD     : IN  std_logic;                                    -- allow storing D value into the register next cycle
            RESET    : IN  std_logic                                     -- reset control
          );
  end component;

  -- *****************************************************************
  -- cl_sub4 : sub 4 to a specifier register ; no carry
  --           This component compute addition by quartet, precomputing
  --           carries for each quartet.

  component cl_sub4 is
     port ( B        : IN  std_logic_vector( 31 downto 0 );              -- input 32b value
            R        : OUT std_logic_vector( 31 downto 0 )               -- output 32b result
          );
  end component;

  -- *****************************************************************
  -- cl_add4 : add 4 to a specifier register ; no carry
  --           This component compute addition by quartet, precomputing
  --           carries for each quartet.
  component cl_add4 is
     port ( B        : IN  std_logic_vector( 31 downto 0 );              -- input 32b value
            R        : OUT std_logic_vector( 31 downto 0 )               -- output 32b result
           );
  end component;

  -- *****************************************************************
  -- cl_add32 : 32 bit adder with Carry
  --
  component cl_add32 is
      port (   Ci       : IN  std_logic;                                   -- carry in
               A        : IN  std_logic_vector( 31 downto 0 );             -- first input 32b value
               B        : IN  std_logic_vector( 31 downto 0 );             -- first input 32b value
               R        : OUT std_logic_vector( 31 downto 0 );             -- output 32b result
               Co       : OUT std_logic                                    -- carry
            );
  end component;


  -- *****************************************************************
  -- cl_full_mul32 : signed/unsigned 32b multiplier
  -- when signed, carry must differ than sign to create an overflow
  -- C != S
  component cl_full_mul32 is
        port ( S : IN  std_logic;                                   -- mode signed (AH)
               A : IN  std_logic_vector( 31 downto  0 );            -- first 32bits operand
               B : IN  std_logic_vector( 31 downto  0 );            -- second 32bits operand
               R : OUT std_logic_vector( 31 downto  0 );            -- result 32bits operand
               C : OUT std_logic                                    -- carry result (to compute overflow)
            );
  end component;

   -- *****************************************************************
   -- cl_div32 : signed/unsigned 32b divider
   component cl_div32 is
     port ( S   : IN  std_logic;                                   -- signed (AH) or not
            A   : IN  std_logic_vector( 31 downto  0 );            -- first 32bits operand   dividend
            B   : IN  std_logic_vector( 31 downto  0 );            -- second 32bits operand  divisor
            D   : OUT std_logic_vector( 31 downto  0 );            -- result 32bits operand
            R   : OUT std_logic_vector( 31 downto  0 );            -- rest 32bits operand
            RDY : OUT std_logic;                                   -- result is ready (AH)
            CLK : IN  std_logic;                                   -- clock signal
            STT : IN  std_logic                                    -- start : set to High to start divider
          );
   end component;

  -- *****************************************************************
  -- cl_buf : three state buffer
  component cl_buf is
     port ( I        : IN  std_ulogic;             -- input
            O        : OUT std_ulogic;             -- output
            C        : IN  std_ulogic              -- control signal (AH)
          );
  end component;


  -- *****************************************************************
  -- cl_bufN : three state N bit buffer
  component cl_bufN is
      generic (  WIDTH   : INTEGER := 32 );                               -- width of the buffer
      port ( I        : IN  std_logic_vector( (WIDTH-1) downto 0 );       -- N bit input
             O        : OUT std_logic_vector( (WIDTH-1) downto 0 );       -- N bit output
             C        : IN  std_logic
            );
  end component;

  -- *****************************************************************
  -- cl_muxN - MxN bits multiplexer
  component cl_mux32 is
      port ( SEL  : IN  STD_LOGIC_VECTOR( 4 downto 0 );
             DIN  : IN  STD_LOGIC_VECTOR( (32*32)-1 downto 0 );
             DOUT : OUT STD_LOGIC_VECTOR( 31 downto 0 )
           );
  end component;

  -- *****************************************************************
  -- cl_muxNx1 - 32x1 MUX
  component cl_mux32x1 is
      port ( SEL  : IN  STD_LOGIC_VECTOR( 4 downto 0 );
             DIN  : IN  STD_LOGIC_VECTOR(31 downto 0 );
             DOUT : OUT STD_LOGIC
           );
  end component;

  -- *****************************************************************
  -- cl_rot32 : 32 bit shit/rotate register with multiple mode and
  --            selectable shift step.
  component cl_rot32 is
      port ( MODE  : IN  std_logic_vector(  2 downto  0 );            -- Mode selection 000 : rot / 010 shift zero /
                                                                        -- 011 shift one / 100 shif carry / 101 sift sign
             Ci    : IN  std_logic;                                   -- carry in
             DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
             STEP  : IN  std_logic_vector(  4 downto  0 );            -- number of step / bit to shift
             I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
             O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
             Co    : OUT std_logic                                    -- output bit
           );
  end component;

  -- ***************************************************************************************
  -- 32 bit decoder ; 5 selection bit width ; copy intput on selected output
  component cl_demux32 is
      port ( SEL  : IN  STD_LOGIC_VECTOR( 4 downto 0 );
             DIN  : IN  STD_LOGIC;
             DOUT : OUT STD_LOGIC_VECTOR( 31 downto 0 )
           );
  end component;


  -- *****************************************************************
  -- cl_7segX - 7 segment codeur for 16 bits hex value for Xilnx board
  --            as segments share only one interface this component have
  --            to maintain value sent and distribute it on the 4 7seg
  --            in time
  component cl_7segX is
      port ( D        : IN  std_logic_vector( 15 downto 0 );              -- data to print on 7seg bloc
             Q        : OUT std_logic_vector(  6 downto 0 );              -- 7seg output leds
             CTRL     : OUT std_logic_vector(  3 downto 0 );              -- 7seg control for multiplexing system
             CLK      : IN  std_logic                                     -- system clock (should be > 1 KHz)
            );
  end component;

  -- *****************************************************************
  -- rom128x32 : ROM component with 128 entries of 32 bits
  --             mapped to a Xilinx specific Unisim component
  type cl_rom128x32_data is array ( 0 to 127 ) of std_logic_vector( 31 downto 0 );
  component cl_rom128x32 is
     generic (
            INIT     : cl_rom128x32_data
              );
     port (
            O        : OUT std_logic_vector( 31 downto 0 );
            A        : IN  std_logic_vector(  6 downto 0 )
          );
  end component;

  -- *****************************************************************
  -- rom64x32 : ROM component with 128 entries of 32 bits
  --            mapped to a Xilinx specific Unisim component
  type cl_rom64x32_data is array ( 0 to 63 ) of std_logic_vector( 31 downto 0 );
  component cl_rom64x32 is
      generic (
               INIT     : cl_rom64x32_data
               );
      port (
               O        : OUT std_logic_vector( 31 downto 0 );
               A        : IN  std_logic_vector(  5 downto 0 )
            );
  end component;

   -- *****************************************************************
   -- ram16x32 : Synchronous RAM component with 16 entries of 32 bits
   --            mapped to a Xilinx specific Unisim component
   component cl_ram16x32 is
      port (
               I        : IN  std_logic_vector( 31 downto 0 );       -- Input data to write
               O        : OUT std_logic_vector( 31 downto 0 );       -- Output read data
               WR0      : IN  std_logic;                             -- Write request for I( 7 downto  0) (AH)
               WR1      : IN  std_logic;                             -- Write request for I(15 downto  8) (AH)
               WR2      : IN  std_logic;                             -- Write request for I(23 downto 16) (AH)
               WR3      : IN  std_logic;                             -- Write request for I(31 downto 24) (AH)
               WCLK     : IN  std_logic;                             -- Write clock
               A        : IN  std_logic_vector(  3 downto 0 )        -- Address.
            );
   end component;

  -- ******************************************************************
  -- Internal function
  function cl_conv_rom128_init ( I : cl_rom128x32_data ; cBit : INTEGER ) return BIT_VECTOR;
  function cl_conv_rom64_init  ( I : cl_rom64x32_data ;  cBit : INTEGER ) return BIT_VECTOR;



end UTILS;




package body UTILS is

   -- ********************************************************************
   -- convert 128 32bits value into a 128 1bit vector for ROM initilizing
   function cl_conv_rom128_init ( I : cl_rom128x32_data ; cBit : INTEGER ) return BIT_VECTOR is
     variable RESULT  : bit_vector (127 downto 0);
   begin
     for iter in RESULT'reverse_range loop
        case I(iter)( cBit ) is
           when '0'     =>  RESULT(iter) := '0';
           when '1'     =>  RESULT(iter) := '1';
           when others  =>  RESULT(iter) := '0';
        end case;
     end loop;
     return RESULT;
   end;
   -- ********************************************************************
   -- convert 64 32bits value into a 128 1bit vector for ROM initilizing
   function cl_conv_rom64_init ( I : cl_rom64x32_data ; cBit : INTEGER ) return BIT_VECTOR is
     variable RESULT  : bit_vector (63 downto 0);
   begin
     for iter in RESULT'reverse_range loop
        case I(iter)( cBit ) is
           when '0'     =>  RESULT(iter) := '0';
           when '1'     =>  RESULT(iter) := '1';
           when others  =>  RESULT(iter) := '0';
        end case;
     end loop;
     return RESULT;
   end;



end UTILS;
