-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    19:32:00 09/07/05
-- Design Name:    utils_add.vhd - different Adder/Substracter component
-- Module Name:    core_lib
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    different component to add, substract registers or values
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *****************************************************************
-- cl_add4 : add 4 to a specifier register ; no carry
--           This component compute addition by quartet, precomputing
--           carries for each quartet.
entity cl_add4 is
     port ( B        : IN  std_logic_vector( 31 downto 0 );             -- input 32b value
            R        : OUT std_logic_vector( 31 downto 0 )              -- output 32b result
          );
end entity cl_add4;


architecture arch_cl_add4 of cl_add4 is
     signal Ci : std_logic_vector(6 downto 0);                          -- internal carry by quartet
begin

     R <= B + 4;                                                        -- compilation result is better :,(

     -- specific first quartet
--     R(0)  <= B(0);
--     R(1)  <= B(1);
--     R(2)  <= NOT ( B(2) );
--     R(3)  <= B(3) XOR B(2);
--     Ci(0) <= B(3) AND B(2);

     -- generic next quartet
--     b0 : for k in 1 to 6 generate
--           Ci(k)     <= B(4*k) AND B(4*k+1) AND B(4*k+2) AND B(4*k+3) AND Ci(k-1);
--     end generate;
--     b1 : for k in 1 to 7 generate
--        R(4*k)    <= B(4*k)   XOR Ci(k-1);
--        R(4*k+1)  <= B(4*k+1) XOR ( B(4*k) AND Ci(k-1) );
--        R(4*k+2)  <= B(4*k+2) XOR ( B(4*k+1) AND B(4*k) AND Ci(k-1) );
--        R(4*k+3)  <= B(4*k+3) XOR ( B(4*k+2) AND B(4*k+1) AND B(4*k) AND Ci(k-1) );
--     end generate;

end arch_cl_add4;

-- #######################################################################################
-- #######################################################################################

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *****************************************************************
-- cl_sub4 : sub 4 to a specifier register ; no carry
--           This component compute addition by quartet, precomputing
--           carries for each quartet.
entity cl_sub4 is
     port ( B        : IN  std_logic_vector( 31 downto 0 );             -- input 32b value
            R        : OUT std_logic_vector( 31 downto 0 )              -- output 32b result
          );
end entity cl_sub4;


architecture arch_cl_sub4 of cl_sub4 is
     signal Ci : std_logic_vector(6 downto 0);                          -- internal carry by quartet
begin

     R <= B - 4;                                                        -- compilation result is better :,(

--     -- specific first quartet
--     R(0)  <= B(0);
--     R(1)  <= B(1);
--     R(2)  <= NOT ( B(2) );
--     R(3)  <= NOT ( B(3) XOR B(2) );
--     Ci(0) <= B(3) OR B(2);

     -- generic next quartet
--     b0 : for k in 1 to 6 generate
--        Ci(k)     <= B(4*k) OR B(4*k+1) OR B(4*k+2) OR B(4*k+3) OR Ci(k-1);
--     end generate;
--     b1 : for k in 1 to 7 generate
--        R(4*k)    <= NOT ( B(4*k) XOR Ci(k-1));
--        R(4*k+1)  <= NOT ( B(4*k+1) XOR ( B(4*k) OR Ci(k-1) ) );
--        R(4*k+2)  <= NOT ( B(4*k+2) XOR ( B(4*k+1) OR B(4*k) OR Ci(k-1) ) );
--        R(4*k+3)  <= NOT ( B(4*k+3) XOR ( B(4*k+2) OR B(4*k+1) OR B(4*k) OR Ci(k-1) ) );
--     end generate;

end arch_cl_sub4;


-- #######################################################################################
-- #######################################################################################

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *****************************************************************
-- cl_add32 : 32 bit adder with Carry
--
entity cl_add32 is
     port ( Ci       : IN  std_logic;                                   -- carry in
            A        : IN  std_logic_vector( 31 downto 0 );             -- first input 32b value
            B        : IN  std_logic_vector( 31 downto 0 );             -- first input 32b value
            R        : OUT std_logic_vector( 31 downto 0 );             -- output 32b result
            Co       : OUT std_logic                                    -- carry
          );
end entity cl_add32;


architecture arch_cl_add32 of cl_add32 is
   signal i_R : std_logic_vector(32 downto 0);
   signal i_A : std_logic_vector(32 downto 0);
   signal i_B : std_logic_vector(32 downto 0);
begin
   -- really simple and fast approach ...
   i_A(32 downto 0) <= A(31) & A (31 downto 0);
   i_B(32 downto 0) <= B(31) & B (31 downto 0);

   i_R <= i_A + i_B + Ci;
   R   <= i_R(31 downto 0);
   Co  <= i_R(32);
end arch_cl_add32;


