-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    09/09/2005
-- Design Name:    utils_board.vhd - Xilinx board specific existing component tools
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    7seg coder
--
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;

-- *****************************************************************
-- cl_7segX - 7 segment codeur for 16 bits hex value for Xilnx board
--            as segments share only one interface this component have
--            to maintain value sent and distribute it on the 4 7seg
--            in time
entity cl_7segX is
     port ( D        : IN  std_logic_vector( 15 downto 0 );             -- data to print on 7seg bloc
            Q        : OUT std_logic_vector(  6 downto 0 );             -- 7seg output leds
            CTRL     : OUT std_logic_vector(  3 downto 0 );             -- 7seg control for multiplexing system
            CLK      : IN  std_logic                                    -- system clock (should be > 1 KHz)
          );
end entity cl_7segX;

architecture arch_cl_7segX of cl_7segX is

  -- *****************************************************************
  -- scREG : Generic register
  component cl_reg
     generic (  WIDTH   : INTEGER := 32 );                               -- width of the register
     port ( D        : IN  std_logic_vector( (WIDTH-1) downto 0 );       -- input data may stored next cycle
            Q        : OUT std_logic_vector( (WIDTH-1) downto 0 );       -- output value of the register
            CLK      : IN  std_logic;                                    -- system clock
            LOAD     : IN  std_logic;                                    -- allow storing D value into the register next cycle
            RESET    : IN  std_logic                                     -- reset control
          );
  end component;

   -- PAD : net Q(6) loc = E14
   --       net Q(5) loc = G13
   --       net Q(4) loc = N15
   --       net Q(3) loc = P15
   --       net Q(2) loc = R16
   --       net Q(1) loc = F13
   --       net Q(0) loc = N16
   --       net CTRL(3) loc = E13
   --       net CTRL(2) loc = F14
   --       net CTRL(1) loc = G14
   --       net CTRL(0) loc = D14
   signal   routed_value : std_logic_vector( 3 downto 0 );             -- 1 digit signal
   signal   i_Q : std_logic_vector( 6 downto 0 );

   type STATE_COUNTER is ( q0, q1, q2, q3 );
   attribute ENUM_ENCODING: STRING;
   attribute ENUM_ENCODING of STATE_COUNTER: type is "0001 0010 0100 1000";
   signal i_STATE, n_STATE : STATE_COUNTER;

begin

   process (CLK)
   begin
      if ( CLK = '1' and CLK'event ) then
       i_STATE <= n_STATE;
      end if;
   end process;

   process ( i_STATE, D )
   begin
     case i_STATE is
      when q0     => n_STATE <= q1;
                     routed_value(3 downto 0) <= D( 3 downto  0);
                     CTRL <= "0111";
      when q1     => n_STATE <= q2;
                     routed_value(3 downto 0) <= D( 7 downto  4);
                     CTRL <= "1110";
      when q2     => n_STATE <= q3;
                     routed_value(3 downto 0) <= D(11 downto  8);
                     CTRL <= "1101";
      when q3     => n_STATE <= q0;
                     routed_value(3 downto 0) <= D(15 downto 12);
                     CTRL <= "1011";
     end case;
   end process;

   i_Q          <= "0000001" when routed_value="0000" else
                   "1001111" when routed_value="0001" else
                   "0010010" when routed_value="0010" else
                   "0000110" when routed_value="0011" else
                   "1001100" when routed_value="0100" else
                   "0100100" when routed_value="0101" else
                   "0100000" when routed_value="0110" else
                   "0001111" when routed_value="0111" else
                   "0000000" when routed_value="1000" else
                   "0000100" when routed_value="1001" else
                   "0001000" when routed_value="1010" else
                   "1100000" when routed_value="1011" else
                   "0110001" when routed_value="1100" else
                   "1000010" when routed_value="1101" else
                   "0110000" when routed_value="1110" else
                   "0111000";

   i_reg7b : cl_reg generic map ( WIDTH => 7 )
                    port map ( D => i_Q, Q => Q, CLK => CLK, LOAD => '1', RESET => '0');



end arch_cl_7segX;


