-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************

--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    19:32:00 09/07/05
-- Design Name:    utils_div.vhd - divider
-- Module Name:    core_lib
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    component to do 32x32 bit divider
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *****************************************************************
-- cl_div32 : signed/unsigned 32 bits divider.
entity cl_div32 is
     port ( S   : IN  std_logic;                                   -- signed (AH) or not
            A   : IN  std_logic_vector( 31 downto  0 );            -- first 32bits operand   dividend
            B   : IN  std_logic_vector( 31 downto  0 );            -- second 32bits operand  divisor
            D   : OUT std_logic_vector( 31 downto  0 );            -- result 32bits operand
            R   : OUT std_logic_vector( 31 downto  0 );            -- rest 32bits operand
            RDY : OUT std_logic;                                   -- result is ready (AH)
            CLK : IN  std_logic;                                   -- clock signal
            STT : IN  std_logic                                    -- start : set to High to start divider
          );
end entity cl_div32;


architecture arch_cl_div32 of cl_div32 is
   signal u_Anot : std_logic_vector( 31 downto  0 );
   signal u_Bnot : std_logic_vector( 31 downto  0 );
   signal u_Ap1  : std_logic_vector( 31 downto  0 );
   signal u_Bp1  : std_logic_vector( 31 downto  0 );
   signal u_A    : std_logic_vector( 32 downto  0 );
   signal u_B    : std_logic_vector( 32 downto  0 );
   signal u_Rnot : std_logic_vector( 31 downto  0 );
   signal u_Rp1  : std_logic_vector( 31 downto  0 );
   signal u_Dnot : std_logic_vector( 31 downto  0 );
   signal u_Dp1  : std_logic_vector( 31 downto  0 );

   signal d_sign : std_logic;

   signal sub1_a : std_logic_vector( 32 downto  0 );
   signal sub1_b : std_logic_vector( 32 downto  0 );
   signal sub1_r : std_logic_vector( 32 downto  0 );
--   signal sub2_r : std_logic_vector( 32 downto  0 );
--   signal sub2_a : std_logic_vector( 33 downto  0 );
--   signal sub2_b : std_logic_vector( 33 downto  0 );
   signal sub3_r : std_logic_vector( 32 downto  0 );
   signal sub3_a : std_logic_vector( 32 downto  0 );
   signal sub3_b : std_logic_vector( 32 downto  0 );
--   signal sub23_r: std_logic_vector( 32 downto  0 );

   signal reg_p  : std_logic_vector( 62 downto  0 );
   signal buf_q  : std_logic_vector( 31 downto  0 );
   signal reg_q  : std_logic_vector( 31 downto  0 );

   signal i_qA   : std_logic;                            -- intermediate result bit low
   signal i_qB   : std_logic;                            -- intermediate result bit high
   signal c_rem  : std_logic_vector( 31 downto 0 );      -- intermediate remainder

   signal p_LOAD  : std_logic;
   signal p_SHIFT : std_logic;
   signal q_INIT  : std_logic;
   signal q_LOADandSHIFT : std_logic;

   type STATE_COUNTER is ( qinit, q0, q1, q2, q3, q4, q5, q6, q7, q8, q9, q10, q11, q12, q13, q14, q15, qend );
   attribute ENUM_ENCODING: STRING;
   attribute ENUM_ENCODING of STATE_COUNTER: type is "000000000000000001 " &
                                                     "000000000000000010 " &
                                                     "000000000000000100 " &
                                                     "000000000000001000 " &
                                                     "000000000000010000 " &
                                                     "000000000000100000 " &
                                                     "000000000001000000 " &
                                                     "000000000010000000 " &
                                                     "000000000100000000 " &
                                                     "000000001000000000 " &
                                                     "000000010000000000 " &
                                                     "000000100000000000 " &
                                                     "000001000000000000 " &
                                                     "000010000000000000 " &
                                                     "000100000000000000 " &
                                                     "001000000000000000 " &
                                                     "010000000000000000 " &
                                                     "100000000000000000 ";
   signal i_STATE, n_STATE : STATE_COUNTER;

begin

   -- Unsign numbers ...
   u_Anot <= not(A);
   u_Bnot <= not(B);
   u_Ap1  <= u_Anot + 1;
   u_Bp1  <= u_Bnot + 1;
   u_A    <= '0' & A when S = '0' OR A(31) = '0' else '0' & u_Ap1;
   u_B    <= '0' & B when S = '0' OR B(31) = '0' else '0' & u_Bp1;

   -- determine output sign
   d_sign <= A(31) xor B(31);    -- 1 when negative


   -- State machine...
   process (CLK)
   begin
      if ( CLK = '1' and CLK'event ) then
       i_STATE <= n_STATE;
      end if;
   end process;

   process ( i_STATE, STT )
   begin
     p_LOAD         <= '0';
     p_SHIFT        <= '0';
     q_INIT         <= '0';
     q_LOADandSHIFT <= '0';
     case i_STATE is
      when qinit  => if ( STT = '1' ) then
                       n_STATE <= q0;
                       RDY     <= '0';
                     else
                       n_STATE <= qinit;
                       RDY     <= '1';
                     end if;
                     p_LOAD  <= '1';
                     q_INIT  <= '1';

      when q0     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b31b30
                     n_STATE <= q1;
                     RDY     <= '0';
      when q1     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b29b28
                     n_STATE <= q2;
                     RDY     <= '0';
      when q2     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b27b26
                     n_STATE <= q3;
                     RDY     <= '0';
      when q3     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b25b24
                     n_STATE <= q4;
                     RDY     <= '0';
      when q4     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b23b22
                     n_STATE <= q5;
                     RDY     <= '0';
      when q5     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b21b20
                     n_STATE <= q6;
                     RDY     <= '0';
      when q6     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b19b18
                     n_STATE <= q7;
                     RDY     <= '0';
      when q7     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b17b16
                     n_STATE <= q8;
                     RDY     <= '0';
      when q8     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b15b14
                     n_STATE <= q9;
                     RDY     <= '0';
      when q9     => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b13b12
                     n_STATE <= q10;
                     RDY     <= '0';
      when q10    => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b11b10
                     n_STATE <= q11;
                     RDY     <= '0';
      when q11    => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b09b08
                     n_STATE <= q12;
                     RDY     <= '0';
      when q12    => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b07b06
                     n_STATE <= q13;
                     RDY     <= '0';
      when q13    => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b05b04
                     n_STATE <= q14;
                     RDY     <= '0';
      when q14    => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b03b02
                     n_STATE <= q15;
                     RDY     <= '0';
      when q15    => p_SHIFT <= '1';
                     q_LOADandSHIFT <= '1';           -- b01b00
                     n_STATE <= qend;
                     RDY     <= '0';

      when qend   => RDY     <= '1';                  -- result is ready
                     n_STATE <= qinit;

      when others => n_STATE <= qinit;
                     RDY     <= '1';                  -- result is ready


     end case;
   end process;

   -- step registers - Dividend P
   process ( CLK, p_LOAD, p_SHIFT, c_rem, u_A  )
   begin
      if ( CLK='1' and CLK'event ) then
         if ( p_LOAD = '1' ) then
            reg_p(32 downto  0) <= u_A(32 downto 0);
            reg_p(62 downto 33) <= (others => '0' );
         else
            if ( p_SHIFT = '1' ) then
                 reg_p(62 downto 31) <= c_rem(31 downto 0);       -- load rest
                 reg_p(30 downto  0) <= reg_p(28 downto 0) & "00";-- shift other bits
            end if;
         end if;
      end if;
   end process;
   -- Result q
   process ( CLK, q_INIT, q_LOADandSHIFT )
   begin
         if ( q_INIT = '1' ) then
            buf_q(31 downto  0) <= (others => '0');
         else
            if ( CLK='1' and CLK'event ) then
               if ( q_LOADandSHIFT = '1' ) then
                 buf_q(31 downto 2) <= reg_q(29 downto 0);
                 buf_q(1) <= i_qA;
                 buf_q(0) <= i_qB;
               end if;
            end if;
         end if;
   end process;
   reg_q <= buf_q;


   sub1_a   <= '0' & reg_p(62 downto 31);
   sub1_b   <= u_B;
   sub1_r   <= sub1_a - sub1_b;                                      -- compute 2*w - d
   i_qA     <= '1' when sub1_r(32) = '0' else '0';                   -- i_qA = 1 when sub1_r >= 0
--   i_qA     <= '1' when sub1_r >= '0' & X"00000000" else '0';        -- q1='1' when 2*w - d > 0


   sub3_a   <= '0' & sub1_r(30 downto 0) & reg_p(30) when i_qA = '1' else  -- compute 4*w ou 2*(2*w -d)
               '0' & reg_p(61 downto 30);
   sub3_b   <= u_B;
   sub3_r   <= sub3_a - sub3_b;
   i_qB     <= '1' when sub3_r(32) = '0' else '0';                   -- i_qB = 1 when sub3_r >= 0
--   i_qB     <= '1' when sub3_r >= '0' & X"00000000" else '0';

   c_rem    <= sub3_r(30 downto 0) & reg_p(29) when i_qB = '1' else
               reg_p(60 downto 29);



   -- operationnal part
--   sub1_a   <= '0' & reg_p( 62 downto 31);
--   sub1_b   <= u_B;
--   sub1_r   <= sub1_a - sub1_b;                                      -- compute 2*w - d
--   i_q1     <= '1' when sub1_r >= '0' & X"00000000" else '0';        -- q1='1' when 2*w - d > 0

--   sub2_a   <= '0' & reg_p ( 62 downto 30 );
--   sub2_b   <= u_B & '0';
--   sub2_r   <= sub2_a - sub2_b;                                      -- compute 4*w - 2*d -> assume sub1_r is true

--   sub3_a   <= '0' & reg_p ( 61 downto 30 );
--   sub3_b   <= u_B;
--   sub3_r   <= sub3_a - sub3_b;                                      -- compute 4*w - d   -> assume sub1_r is false
--   sub23_r  <= sub2_r(32 downto 0) when i_q1 = '1' else sub3_r;      -- select the good choice
--   i_q0     <= '1' when sub23_r >= '0' & X"00000000" else '0';       -- q0='1' when choice is > 0
--   c_rem    <= sub23_r(31 downto 0);

   -- output interface
   u_Dnot <= not(reg_q);
   u_Rnot <= not(reg_p(62 downto 31));
   u_Dp1  <= u_Dnot + 1;
   u_Rp1  <= u_Rnot + 1;
   D      <= reg_q when S = '0' OR d_sign = '0' else u_Dp1;
   R      <= reg_p(62 downto 31) when S = '0' OR d_sign = '0' else u_Rp1;

end arch_cl_div32;




