-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    19:32:00 09/07/05
-- Design Name:    utils_mul.vhd - multiplier
-- Module Name:    core_lib
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    component to do 32x32 bit multiply
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *****************************************************************
-- cl_mul33 : signed 33 bits multiplier. Return a 66 bit value
entity cl_mul33 is
     port ( A : IN  std_logic_vector( 32 downto  0 );            -- first 33bits operand
            B : IN  std_logic_vector( 32 downto  0 );            -- second 33bits operand
            R : OUT std_logic_vector( 65 downto  0 )             -- result 32bits operand
          );
end entity cl_mul33;


architecture arch_cl_mul33 of cl_mul33 is
begin

   R <= A * B;

end arch_cl_mul33;


-- ############################################################################################"
-- ############################################################################################"


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- *****************************************************************
-- cl_full_mul32 : signed/unsigned 32b multiplier
-- This implementation uses a 33 bit signed multiplier and convert the
-- 32 bit signed / unsigned value into a 33b signed value.
-- Unsigned carry = 1 when result is higher than a 32b unsigned value
-- Signed carry is computed depending on sign to have Overflow as C!=S
-- active when the 32bits signed results is wrong
entity cl_full_mul32 is
     port ( S : IN  std_logic;                                   -- mode signed (AH)
            A : IN  std_logic_vector( 31 downto  0 );            -- first 32bits operand
            B : IN  std_logic_vector( 31 downto  0 );            -- second 32bits operand
            R : OUT std_logic_vector( 31 downto  0 );            -- result 32bits operand
            C : OUT std_logic                                    -- carry result (to compute overflow)
          );
end entity cl_full_mul32;

architecture arch_cl_full_mul32 of cl_full_mul32 is

   component cl_mul33 is
        port ( A : IN  std_logic_vector( 32 downto  0 );            -- first 33bits operand
               B : IN  std_logic_vector( 32 downto  0 );            -- second 33bits operand
               R : OUT std_logic_vector( 65 downto  0 )             -- result 32bits operand
              );
   end component;

   signal r_c        : std_logic;                                 -- carry returned from multiplier
   signal r_c_signed : std_logic;                                 -- carry returned from multiplier recomputer for signed mode

   signal a_signed   : std_logic_vector(32 downto 0);             -- convert 32bit signed / unsigned source into a 33bit signed value
   signal b_signed   : std_logic_vector(32 downto 0);             -- convert 32bit signed / unsigned source into a 33bit signed value
   signal r_signed   : std_logic_vector(65 downto 0);             -- 65 bit signed result

begin

   -- compute result
   a_signed(31 downto 0) <= A(31 downto 0);
   a_signed(32) <= A(31) when S = '1' else '0';
   b_signed(31 downto 0) <= B(31 downto 0);
   b_signed(32) <= A(31) when S = '1' else '0';
   mul1 : cl_mul33 port map ( A => a_signed, B => b_signed, R => r_signed);
   R <= r_signed(31 downto 0);


   -- compute carry
   r_c        <= '0' when r_signed(65 downto 32) = X"00000000" else '1';
   r_c_signed <= not(r_signed(31)) when r_signed(65 downto 31) = X"FFFFFFFF" & '1' else
                 not(r_signed(31)) when r_signed(65 downto 31) = X"00000000" & '0' else
                 r_signed(31);
   C <=  r_c when S = '0' else  r_c_signed;

end arch_cl_full_mul32;
