-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    13:29 10/11/2005
-- Design Name:    util_ram.vhd - RAM component
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  Xilinx dependent
-- Tool versions:
-- Description:    128x32 bits Rom device maped to a Xilinx UNISIM component
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

library core_lib;
use core_lib.utils.ALL;


-- *****************************************************************
-- ram16x32 : Synchronous RAM component with 16 entries of 32 bits
--            mapped to a Xilinx specific Unisim component
entity cl_ram16x32 is
     port (
            I        : IN  std_logic_vector( 31 downto 0 );       -- Input data to write
            O        : OUT std_logic_vector( 31 downto 0 );       -- Output read data
            WR0      : IN  std_logic;                             -- Write request for I( 7 downto  0) (AH)
            WR1      : IN  std_logic;                             -- Write request for I(15 downto  8) (AH)
            WR2      : IN  std_logic;                             -- Write request for I(23 downto 16) (AH)
            WR3      : IN  std_logic;                             -- Write request for I(31 downto 24) (AH)
            WCLK     : IN  std_logic;                             -- Write clock
            A        : IN  std_logic_vector(  3 downto 0 )        -- Address.
          );
end entity cl_ram16x32;


architecture arch_cl_ram16x32 of cl_ram16x32 is

begin

   g0 : for k in 31 downto 24 generate
      g_ram0 : RAM16X1S port map( D => I(k), WE => WR3, WCLK => WCLK, A0 => A(0), A1 => A(1), A2 => A(2), A3 => A(3), O => O(k) );
   end generate;
   g1 : for k in 23 downto 16 generate
      g_ram1 : RAM16X1S port map( D => I(k), WE => WR2, WCLK => WCLK, A0 => A(0), A1 => A(1), A2 => A(2), A3 => A(3), O => O(k) );
   end generate;
   g2 : for k in 15 downto 8 generate
      g_ram2 : RAM16X1S port map( D => I(k), WE => WR1, WCLK => WCLK, A0 => A(0), A1 => A(1), A2 => A(2), A3 => A(3), O => O(k) );
   end generate;
   g3 : for k in 7 downto 0 generate
      g_ram3 : RAM16X1S port map( D => I(k), WE => WR0, WCLK => WCLK, A0 => A(0), A1 => A(1), A2 => A(2), A3 => A(3), O => O(k) );
   end generate;

end arch_cl_ram16x32;

