-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    20:39:36 08/11/05
-- Design Name:    registers.vhd - registers basic library
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    standard elements to build registers, may be latter map on
--                 target optimized one.
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library core_lib;
use core_lib.utils.ALL;


-- *****************************************************************
-- scREG : Generic register
entity cl_reg is
     generic (  WIDTH   : INTEGER := 32 );                               -- width of the register
     port ( D        : IN  std_logic_vector( (WIDTH-1) downto 0 );       -- input data may stored next cycle
            Q        : OUT std_logic_vector( (WIDTH-1) downto 0 );       -- output value of the register
            CLK      : IN  std_logic;                                    -- system clock
            LOAD     : IN  std_logic;                                    -- allow storing D value into the register next cycle
            RESET    : IN  std_logic                                     -- reset control
          );
end entity cl_reg;


architecture arch_cl_reg of cl_reg is

   signal   i_reg : std_logic_vector( (WIDTH-1) downto 0 );

--   component FDCE
--     port ( Q        : OUT std_logic;                                    -- flip flop output bit
---            D        : IN  std_logic;                                    -- bit to load into flip flop when CE will be enabled
 --           C        : IN  std_logic;                                    -- clock
  --          CE       : IN  std_logic;                                    -- chip enable - allow D value to be loaded synchronously with CLK, aticve high
   --         CLR      : IN  std_logic                                     -- clear internal value to zero, active high
--           );
--   end component;

begin

   process(CLK, RESET)
   begin
      if ( RESET = '1' ) then
         i_reg <= (OTHERS => '0');
      elsif ( CLK = '1' and CLK'EVENT ) then
        if ( LOAD = '1' ) then
           i_reg <= D;
        end if;
      end if;
   end process;
   Q <= i_reg;

--  b0 : for k in 0 to (WIDTH-1) generate
--     i_fdce : cl_fdce port map ( Q => Q(k), D => D(k), C => CLK, CE => LOAD, CLR => RESET );
--  end generate;

end arch_cl_reg;
