-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    22:27:12 09/26/05
-- Design Name:    util_rom.vhd - ROM component
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  Xilinx dependent
-- Tool versions:
-- Description:    128x32 bits Rom device maped to a Xilinx UNISIM component
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

library core_lib;
use core_lib.utils.ALL;


-- *****************************************************************
-- rom128x32 : ROM component with 128 entries of 32 bits
--             mapped to a Xilinx specific Unisim component
entity cl_rom128x32 is
     generic (
            INIT     : cl_rom128x32_data
            );
     port (
            O        : OUT std_logic_vector( 31 downto 0 );
            A        : IN  std_logic_vector(  6 downto 0 )
          );
end entity cl_rom128x32;


architecture arch_cl_rom128x32 of cl_rom128x32 is
begin

   g2 : for m in 31 downto 0 generate

      ROM128X1_inst : ROM128X1
      generic map (
         INIT => cl_conv_rom128_init( INIT,m )
      )
      port map (
         O => O(m),
         A0 => A(0), -- ROM address[0]
         A1 => A(1), -- ROM address[1]
         A2 => A(2), -- ROM address[2]
         A3 => A(3), -- ROM address[3]
         A4 => A(4), -- ROM address[4]
         A5 => A(5), -- ROM address[5]
         A6 => A(6)  -- ROM address[6]
      );

   end generate;

end arch_cl_rom128x32;

-- ###################################################################
-- ###################################################################

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

library core_lib;
use core_lib.utils.ALL;


-- *****************************************************************
-- rom64x32 : ROM component with 128 entries of 32 bits
--            mapped to a Xilinx specific Unisim component
entity cl_rom64x32 is
     generic (
            INIT     : cl_rom64x32_data
            );
     port (
            O        : OUT std_logic_vector( 31 downto 0 );
            A        : IN  std_logic_vector(  5 downto 0 )
          );
end entity cl_rom64x32;


architecture arch_cl_rom64x32 of cl_rom64x32 is
begin

   g2 : for m in 31 downto 0 generate

      ROM64X1_inst : ROM64X1
      generic map (
         INIT => cl_conv_rom64_init( INIT,m )
      )
      port map (
         O => O(m),
         A0 => A(0), -- ROM address[0]
         A1 => A(1), -- ROM address[1]
         A2 => A(2), -- ROM address[2]
         A3 => A(3), -- ROM address[3]
         A4 => A(4), -- ROM address[4]
         A5 => A(5)  -- ROM address[5]
      );

   end generate;

end arch_cl_rom64x32;

