-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    19:32:00 09/07/05
-- Design Name:    utils_rot.vhd - register shift / rotation bloc
-- Module Name:    core_lib
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    components to perform 16, 8, 4, 2, 1 bit shifts
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *****************************************************************
-- cl_rot16 : 16 bits shift / rotation
entity cl_rot16 is
     port ( IBIT  : IN  std_logic;                                   -- injected bit
            PS    : IN  std_logic;                                   -- perform operation (AH) / O=I (AL)
            ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
            DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
            I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
            O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
            C     : OUT std_logic                                    -- output bit
          );
end entity cl_rot16;


architecture arch_cl_rot16 of cl_rot16 is
   constant DIR_LEFT  : std_logic := '0';
   constant DIR_RIGHT : std_logic := '1';

   signal   o_rot : std_logic_vector(31 downto 0);
   signal   o_sft : std_logic_vector(31 downto 0);
   signal   o_tmp : std_logic_vector(31 downto 0);
   signal   i_val : std_logic_vector(15 downto 0);
begin
   g0 : for i in 15 downto 0 generate
      i_val(i)         <= IBIT;
   end generate;

   o_rot(31 downto 16) <= I(15 downto  0);
   o_rot(15 downto  0) <= I(31 downto 16);

   o_sft(31 downto 16) <= I(15 downto  0) when DIR = DIR_LEFT  else i_val;
   o_sft(15 downto  0) <= I(31 downto 16) when DIR = DIR_RIGHT else i_val;

   o_tmp <= o_rot when ROT = '1' else o_sft;
   O     <= o_tmp when PS  = '1' else I;

   C     <= I(15) when DIR = DIR_RIGHT else I(16);
end arch_cl_rot16;


-- ############################################################################################"
-- ############################################################################################"

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- *****************************************************************
-- cl_rot8 : 8 bits shift / rotation
entity cl_rot8 is
     port ( IBIT  : IN  std_logic;                                   -- injected bit
            PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
            ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
            DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
            I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
            O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
            C     : OUT std_logic                                    -- output bit
          );
end entity cl_rot8;


architecture arch_cl_rot8 of cl_rot8 is
   constant DIR_LEFT  : std_logic := '0';
   constant DIR_RIGHT : std_logic := '1';

   signal   o_rot : std_logic_vector(31 downto 0);
   signal   o_sft : std_logic_vector(31 downto 0);
   signal   o_tmp : std_logic_vector(31 downto 0);
   signal   i_val : std_logic_vector( 7 downto 0);

begin
   g0 : for i in 7 downto 0 generate
      i_val(i)         <= IBIT;
   end generate;

   o_rot(31 downto 24) <= I(23 downto 16) when DIR = DIR_LEFT else I( 7 downto  0);
   o_rot(23 downto  8) <= I(15 downto  0) when DIR = DIR_LEFT else I(31 downto 16);
   o_rot( 7 downto  0) <= I(31 downto 24) when DIR = DIR_LEFT else I(15 downto  8);

   o_sft(31 downto 24) <= I(23 downto 16)   when DIR = DIR_LEFT else i_val( 7 downto  0);
   o_sft(23 downto  8) <= I(15 downto  0)   when DIR = DIR_LEFT else     I(31 downto 16);
   o_sft( 7 downto  0) <= i_val(7 downto 0) when DIR = DIR_LEFT else     I(15 downto  8);

   o_tmp <= o_rot when ROT = '1' else o_sft;
   O     <= o_tmp when PS  = '1' else I;

   C     <= I(7) when DIR = DIR_RIGHT else I(24);
end arch_cl_rot8;


-- ############################################################################################"
-- ############################################################################################"

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- *****************************************************************
-- cl_rot4 : 4 bits shift / rotation
entity cl_rot4 is
     port ( IBIT  : IN  std_logic;                                   -- injected bit
            PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
            ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
            DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
            I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
            O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
            C     : OUT std_logic                                    -- output bit
          );
end entity cl_rot4;


architecture arch_cl_rot4 of cl_rot4 is
   constant DIR_LEFT  : std_logic := '0';
   constant DIR_RIGHT : std_logic := '1';

   signal   o_rot : std_logic_vector(31 downto 0);
   signal   o_sft : std_logic_vector(31 downto 0);
   signal   o_tmp : std_logic_vector(31 downto 0);
   signal   i_val : std_logic_vector( 3 downto 0);

begin
   g0 : for i in 3 downto 0 generate
      i_val(i)         <= IBIT;
   end generate;

   o_rot(31 downto 28) <= I(27 downto 24) when DIR = DIR_LEFT else I( 3 downto  0);
   o_rot(27 downto  4) <= I(23 downto  0) when DIR = DIR_LEFT else I(31 downto  8);
   o_rot( 3 downto  0) <= I(31 downto 28) when DIR = DIR_LEFT else I( 7 downto  4);

   o_sft(31 downto 28) <= I(27 downto 24)   when DIR = DIR_LEFT else i_val( 3 downto  0);
   o_sft(27 downto  4) <= I(23 downto  0)   when DIR = DIR_LEFT else     I(31 downto  8);
   o_sft( 3 downto  0) <= i_val(3 downto 0) when DIR = DIR_LEFT else     I( 7 downto  4);

   o_tmp <= o_rot when ROT = '1' else o_sft;
   O     <= o_tmp when PS  = '1' else I;

   C     <= I(28) when DIR = DIR_RIGHT else I(3);

end arch_cl_rot4;


-- ############################################################################################"
-- ############################################################################################"

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- *****************************************************************
-- cl_rot2 : 2 bits shift / rotation
entity cl_rot2 is
     port ( IBIT  : IN  std_logic;                                   -- injected bit
            PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
            ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
            DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
            I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
            O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
            C     : OUT std_logic                                    -- output bit
          );
end entity cl_rot2;

architecture arch_cl_rot2 of cl_rot2 is
   constant DIR_LEFT  : std_logic := '0';
   constant DIR_RIGHT : std_logic := '1';

   signal   o_rot : std_logic_vector(31 downto 0);
   signal   o_sft : std_logic_vector(31 downto 0);
   signal   o_tmp : std_logic_vector(31 downto 0);
   signal   i_val : std_logic_vector( 1 downto 0);

begin
   g0 : for i in 1 downto 0 generate
      i_val(i)         <= IBIT;
   end generate;

   o_rot(31 downto 30) <= I(29 downto 28) when DIR = DIR_LEFT else I( 1 downto  0);
   o_rot(29 downto  2) <= I(27 downto  0) when DIR = DIR_LEFT else I(31 downto  4);
   o_rot( 1 downto  0) <= I(31 downto 30) when DIR = DIR_LEFT else I( 3 downto  2);

   o_sft(31 downto 30) <= I(29 downto 28)   when DIR = DIR_LEFT else i_val( 1 downto  0);
   o_sft(29 downto  2) <= I(27 downto  0)   when DIR = DIR_LEFT else     I(31 downto  4);
   o_sft( 1 downto  0) <= i_val(1 downto 0) when DIR = DIR_LEFT else     I( 3 downto  2);

   o_tmp <= o_rot when ROT = '1' else o_sft;
   O     <= o_tmp when PS  = '1' else I;

   C     <= I(30) when DIR = DIR_RIGHT else I(1);

end arch_cl_rot2;


-- ############################################################################################"
-- ############################################################################################"

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- *****************************************************************
-- cl_rot1 : 1 bits shift / rotation
entity cl_rot1 is
     port ( IBIT  : IN  std_logic;                                   -- injected bit
            PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
            ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
            DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
            I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
            O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
            C     : OUT std_logic                                    -- output bit
          );
end entity cl_rot1;

architecture arch_cl_rot1 of cl_rot1 is
   constant DIR_LEFT  : std_logic := '0';
   constant DIR_RIGHT : std_logic := '1';

   signal   o_rot : std_logic_vector(31 downto 0);
   signal   o_sft : std_logic_vector(31 downto 0);
   signal   o_tmp : std_logic_vector(31 downto 0);

begin

   o_rot(31)           <= I(30)           when DIR = DIR_LEFT else I(0);
   o_rot(30 downto  1) <= I(29 downto  0) when DIR = DIR_LEFT else I(31 downto  2);
   o_rot(0)            <= I(31)           when DIR = DIR_LEFT else I(1);

   o_sft(31)           <= I(30)           when DIR = DIR_LEFT else IBIT;
   o_sft(30 downto  1) <= I(29 downto  0) when DIR = DIR_LEFT else I(31 downto  2);
   o_sft(0)            <= IBIT            when DIR = DIR_LEFT else I(1);

   o_tmp <= o_rot when ROT = '1' else o_sft;
   O     <= o_tmp when PS  = '1' else I;

   C     <= I(31) when DIR = DIR_RIGHT else I(0);

end arch_cl_rot1;



-- ############################################################################################"
-- ############################################################################################"

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- *****************************************************************
-- cl_rot32 : 32 bit shit/rotate register with multiple mode and
--            selectable shift step.
entity cl_rot32 is
     port ( MODE  : IN  std_logic_vector(  2 downto  0 );            -- Mode selection 000 : rot / 010 shift zero /
                                                                     -- 011 shift one / 100 shif carry / 101 sift sign
            Ci    : IN  std_logic;                                   -- carry in
            DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
            STEP  : IN  std_logic_vector(  4 downto  0 );            -- number of step / bit to shift
            I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
            O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
            Co    : OUT std_logic                                    -- output bit
          );
end entity cl_rot32;

architecture arch_cl_rot32 of cl_rot32 is

   component cl_rot16 is
      port ( IBIT  : IN  std_logic;                                   -- injected bit
               PS    : IN  std_logic;                                   -- perform operation (AH) / O=I (AL)
               ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
               DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
               I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
               O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
               C     : OUT std_logic                                    -- output bit
            );
   end component;


   component cl_rot8 is
      port ( IBIT  : IN  std_logic;                                   -- injected bit
               PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
               ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
               DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
               I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
               O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
               C     : OUT std_logic                                    -- output bit
            );
   end component;


   component cl_rot4 is
      port ( IBIT  : IN  std_logic;                                   -- injected bit
               PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
               ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
               DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
               I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
               O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
               C     : OUT std_logic                                    -- output bit
            );
   end component;

   component cl_rot2 is
      port ( IBIT  : IN  std_logic;                                   -- injected bit
               PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
               ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
               DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
               I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
               O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
               C     : OUT std_logic                                    -- output bit
            );
   end component;

   component cl_rot1 is
      port ( IBIT  : IN  std_logic;                                   -- injected bit
               PS    : IN  std_logic;                                   -- perform shift (AH) / O=I (AL)
               ROT   : IN  std_logic;                                   -- perform a rotation (AH) / shift (AL)
               DIR   : IN  std_logic;                                   -- Right = 1 / left = 0
               I     : IN  std_logic_vector( 31 downto  0 );            -- 32 bit input value to shift
               O     : OUT std_logic_vector( 31 downto  0 );            -- 32 bit output value shifted
               C     : OUT std_logic                                    -- output bit
            );
   end component;


   constant DIR_LEFT       : std_logic := '0';
   constant DIR_RIGHT      : std_logic := '1';
   constant MODE_ROT       : std_logic_vector(2 downto 0) := "000";
   constant MODE_SFT_ZERO  : std_logic_vector(2 downto 0) := "010";
   constant MODE_SFT_ONE   : std_logic_vector(2 downto 0) := "011";
   constant MODE_SFT_CARRY : std_logic_vector(2 downto 0) := "100";
   constant MODE_SFT_SIGN  : std_logic_vector(2 downto 0) := "101";

   signal   o_rot16 : std_logic_vector(31 downto 0);
   signal   o_rot8  : std_logic_vector(31 downto 0);
   signal   o_rot4  : std_logic_vector(31 downto 0);
   signal   o_rot2  : std_logic_vector(31 downto 0);
   signal   o_rot1  : std_logic_vector(31 downto 0);

   signal   c_16    : std_logic;
   signal   c_8     : std_logic;
   signal   c_4     : std_logic;
   signal   c_2     : std_logic;
   signal   c_1     : std_logic;

   signal   i_bit   : std_logic;
   signal   i_rot   : std_logic;

begin

   i_bit <= '1'   when MODE = MODE_SFT_ONE   else
            Ci    when MODE = MODE_SFT_CARRY else
            I(31) when MODE = MODE_SFT_SIGN  else
            '0';
   i_rot <= '1'   when MODE = MODE_ROT else '0';

   r16 : cl_rot16 port map ( IBIT => i_bit, PS => STEP(4), ROT => i_rot, DIR => DIR, I => I,       O => o_rot16, C => c_16 );
   r8  : cl_rot8  port map ( IBIT => i_bit, PS => STEP(3), ROT => i_rot, DIR => DIR, I => o_rot16, O => o_rot8,  C => c_8  );
   r4  : cl_rot4  port map ( IBIT => i_bit, PS => STEP(2), ROT => i_rot, DIR => DIR, I => o_rot8,  O => o_rot4,  C => c_4  );
   r2  : cl_rot2  port map ( IBIT => i_bit, PS => STEP(1), ROT => i_rot, DIR => DIR, I => o_rot4,  O => o_rot2,  C => c_2  );
   r1  : cl_rot1  port map ( IBIT => i_bit, PS => STEP(0), ROT => i_rot, DIR => DIR, I => o_rot2,  O => o_rot1,  C => c_1  );

   O  <= o_rot1;
   Co <= c_1  when STEP(0) = '1' else
         c_2  when STEP(1) = '1' else
         c_4  when STEP(2) = '1' else
         c_8  when STEP(3) = '1' else
         c_16 when STEP(4) = '1' else
         '0';

end arch_cl_rot32;
