-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    19:47:00 09/23/05
-- Design Name:    alu.vhdl - SC91 alu
--                            http://www.simple-cpu.com/instruction-chargement-valeur-directe.php
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    simple-cpu arithmetical and logical unit
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library core_lib;
use core_lib.utils.ALL;

library work;
use work.instruction_set.ALL;


-- *****************************************************************************
-- ALU ENTITY DESCRIPTION
-- *****************************************************************************
entity sc91_alu is
   port (

        -- Instruction data input - instruction bit 27 downto 0 could contain interesting information
        IVV       : IN   std_logic_vector(31 downto 0);    -- instruction bit for mode and direct data

        -- Data buses
        DNN       : IN   std_logic_vector(31 downto 0);    -- first source operand data bus
        DMM       : IN   std_logic_vector(31 downto 0);    -- second source operand data bus
        DDD       : OUT  std_logic_vector(31 downto 0);    -- destination operand data bus

        -- Specific flag signals
        C         : OUT  std_logic;                         -- carry signal from ALU
        FL        : IN   std_logic_vector(15 downto 0);     -- computed flags for ALU and Instruction

        -- Stack signal
        SP        : IN  std_logic_vector(31 downto 0);      -- SP Register value for PUSH/POP/CALL instructions

        -- Specific control signals
        ALU_SELECT: IN   t_recoding;                        -- Indicate to ALU what output to choose (instruction recoding)
        ALU_ROUTE : IN   t_route;                           -- choose ALU output depending on instruction
        CLK       : IN   std_logic;                         -- clock signal for division
        ALU_RDY   : OUT  std_logic;                         -- requested result is ready


        -- Memory interface
        DATA_A    : OUT  std_logic_vector(31 downto 0);    -- Data memory interface : address
        DATA_O    : OUT  std_logic_vector(31 downto 0);    -- Data memory interface : data to write into memory
        DATA_WE   : OUT  std_logic_vector( 3 downto 0);    -- Indicate Write Len (1, 2 or 4 bytes = 0001 0011 1111)
        DATA_RE   : OUT  std_logic_vector( 3 downto 0);    -- Indicate Read  Len (1, 2 or 4 bytes = 0001 0011 1111)
        DATA_I    : IN   std_logic_vector(31 downto 0);    -- Data memory interface : data to read into memory

        -- not use
        NU        : OUT std_logic                          -- not used, just that last line doesn't need ";"

   );
end sc91_alu;

architecture arch_sc91_alu of sc91_alu is

   -- LOADd
   signal LOADd_mode : std_logic_vector(1  downto 0);   -- LOADd mode signal
   signal LOADd_out  : std_logic_vector(31 downto 0);   -- LOADd output signal

   -- LOADr / LAODm
   signal LOADx_mode : std_logic_vector(2 downto 0);    -- LOADr mode signal
   signal LOADx_out  : std_logic_vector(31 downto 0);   -- LOADr output signal
   signal LOADx_in   : std_logic_vector(31 downto 0);   -- LOADr input signal from DNN or DATA_I
   signal LOADx_re   : std_logic_vector(3 downto 0);    -- Memory interface read enable
   signal LOADx_is   : std_logic;

   -- AND
   signal AND_out    : std_logic_vector(31 downto 0);   -- AND output signam
   -- OR
   signal OR_out     : std_logic_vector(31 downto 0);   -- OR output signam
   -- XOR
   signal XOR_out    : std_logic_vector(31 downto 0);   -- XOR output signam

   -- JB
   signal JB_bitSel  : std_logic;                       -- route the specified bit compared
   signal JB_bitRes  : std_logic;                       -- result of the specific test bit result
   signal JB_combRes : std_logic_vector(3 downto 0);    -- combined result between bitRes and specific mode
   signal JB_globRes : std_logic;                       -- global result for the test
   signal JB_valToadd : std_logic_vector(31 downto 0);  -- value to add to R00 if test is positive
   signal JB_vBit    : std_logic;                       -- bit value to compare with.

   -- CPYB
   signal CPYB_bitSel : std_logic;                      -- selected source bit
   signal CPYB_mode   : std_logic_vector( 1 downto 0);  -- CPYB mode bits
   signal CPYB_out    : std_logic_vector(31 downto 0);  -- output for CPYB
   signal CPYB_C      : std_logic;                      -- outup carry = readed bit

   -- ADDr
   signal ADDR_mode   : std_logic_vector( 2 downto 0);  -- ADDr mode bits

   -- ADDd
   signal ADDD_value  : std_logic_vector(31 downto 0);  -- ADDd direct value once extended

   -- MUL
   signal MUL_C       : std_logic;                      -- MULTIPLIER CARRY OUT
   signal MUL_mode    : std_logic;                      -- MULTIPLIER MODE (High equal to signed)
   signal MUL_out     : std_logic_vector(31 downto 0);  -- MULTIPLIER OUTPUT

   -- DIV
   signal DIV_sign    : std_logic;                      -- DIV signed mode or not
   signal DIV_c       : std_logic;                      -- DIV carry
   signal DIV_cr      : std_logic;                      -- Intermediate carry
   signal DIV_cd      : std_logic;                      -- Intermediate carry
   signal DIV_cg      : std_logic;                      -- Intermediate carry
   signal DIV_res     : std_logic_vector(31 downto 0);  -- DIV result
   signal DIV_rest    : std_logic_vector(31 downto 0);  -- DIV rest
   signal DIV_out     : std_logic_vector(31 downto 0);  -- DIV result
   signal DIV_rdy     : std_logic;                      -- ALU result is ready

   -- ROT
   signal ROT_C       : std_logic;                      -- SHIFT / ROTATE CARRY OUT
   signal ROT_out     : std_logic_vector(31 downto 0);  -- SHIFT / ROTATE OUTPUT

   -- *** ADDER ***
   signal ADD_Ci      : std_logic;                      -- ADDER/SUBSTRACTER Carry In
   signal ADD_Co      : std_logic;                      -- ADDER/SUBSTRACTER Carry Out
   signal ADD_A       : std_logic_vector(31 downto 0);  -- ADDER/SUBSTRACTER first operand
   signal ADD_B       : std_logic_vector(31 downto 0);  -- ADDER/SUBSTRACTER second operand
   signal ADD_out     : std_logic_vector(31 downto 0);  -- ADDER/SUBSTRACTER result

   -- ==========================================================================================
   -- MISC SIGNALS
   signal i_DNN      : std_logic_vector(31 downto 0);   -- DNN, may be load with a direct Data in case of LOADd

begin
   -- ==========================================================================================
   -- DNN Specific routing
   i_DNN(15 downto 0)  <= IVV(IVV_LOADd_VALUE_TOP downto IVV_LOADd_VALUE_TOP-15) when ALU_SELECT(SELECT_LOADD) = '1' else
                          DNN(15 downto 0);
   i_DNN(31 downto 16) <= "0000000000000000" when ALU_SELECT(SELECT_LOADD) = '1' else
                          DNN(31 downto 16);

   -- ==========================================================================================
   -- LOADd Instruction block
   LOADd_MODE(1 downto 0) <= IVV(IVV_LOADd_MODE_TOP downto IVV_LOADd_MODE_TOP-1);
   g0: for i in 31 downto 16 generate
      LOADd_out(i) <= i_DNN(i-16) when LOADd_mode = LOADd_MODE_HIGH   else
                      i_DNN(15)   when LOADd_mode = LOADd_MODE_SIGNED else
                      DMM(i)      when LOADd_mode = LOADd_MODE_LOW    else
                      '0';
   end generate;
   g1 : for i in 15 downto 0 generate
      LOADd_out(i) <= DMM(i) when LOADd_mode = LOADd_MODE_HIGH else
                      i_DNN(i);
   end generate;

   -- ==========================================================================================
   -- LOADr / LOADm / POP Instruction block
   LOADx_is <= ALU_SELECT(SELECT_LOADM) OR ALU_SELECT(SELECT_POP);
   LOADx_in <= DATA_I when LOADx_is = '1' else DNN;
   LOADx_MODE(2 downto 0) <= IVV(IVV_LOADx_MODE_TOP downto IVV_LOADx_MODE_TOP-2);
   process (LOADx_MODE, LOADx_IN)
   begin
      case LOADx_MODE is
         when LOADx_MODE_8S =>   LOADx_OUT( 7 downto  0) <=  LOADx_IN( 7 downto  0);
                                 LOADx_OUT(31 downto  8) <= ( others => LOADx_IN( 7) );
                                 LOADx_RE  <= "0001";

         when LOADx_MODE_8U =>   LOADx_OUT( 7 downto  0) <=  LOADx_IN( 7 downto  0);
                                 LOADx_OUT(31 downto  8) <= ( others => '0' );
                                 LOADx_RE  <= "0001";

         when LOADx_MODE_16S =>  LOADx_OUT(15 downto  0) <=  LOADx_IN(15 downto  0);
                                 LOADx_OUT(31 downto 16) <= ( others => LOADx_IN(15) );
                                 LOADx_RE  <= "0011";

         when LOADx_MODE_16U =>  LOADx_OUT(15 downto  0) <=  LOADx_IN(15 downto  0);
                                 LOADx_OUT(31 downto 16) <= ( others => '0' );
                                 LOADx_RE  <= "0011";

         when others =>          LOADx_OUT(31 downto  0) <=  LOADx_IN(31 downto  0);
                                 LOADx_RE  <= "1111";
      end case;
   end process;
   DATA_RE <= LOADx_RE when ALU_SELECT(SELECT_LOADM)='1' OR ALU_SELECT(SELECT_POP) = '1' else "0000";


   -- ==========================================================================================
   -- Standard LOGIC Instruction block (AND/OR/XOR)
   g2 : for i in 31 downto 0 generate
      AND_out(i) <= DNN(i) AND DMM(i);
      OR_out(i)  <= DNN(i) OR  DMM(i);
      XOR_out(i) <= DNN(i) XOR DMM(i);
   end generate;

   -- ==========================================================================================
   -- Conditional Jump JB
   --    > RMM receive R00 / RNN receive register to test / RDD receive RMM+IVV RELATIVE*4 is test
   --      is ok

   -- Compute test basic based on a single bit test
   jb1 : cl_mux32x1 port map ( SEL  => IVV(IVV_JB_BIT_TOP downto IVV_JB_BIT_TOP-4),
                               DIN  => DNN,
                               DOUT => JB_bitSel
                             );
   JB_vBit   <= IVV(IVV_JB_V_TOP);
   JB_bitRes <= NOT(JB_bitSel XOR JB_vBit);

   -- Compute specific mode combined test
   JB_combRes(0) <= JB_bitRes OR FL(FLAG_Z);                             -- bit or Z=1
   JB_combRes(1) <= JB_bitRes AND NOT(FL(FLAG_Z));                       -- bit and Z=0
   JB_combRes(2) <= JB_bitRes AND NOT((FL(FLAG_S32) XOR FL(FLAG_O32)));  -- bit and S=O
   JB_combRes(3) <= JB_bitRes OR (FL(FLAG_S32) XOR FL(FLAG_O32));        -- bit or S!=O

   JB_globRes <= JB_combRes(0) when ( IVV(IVV_JB_MODE_TOP downto IVV_JB_MODE_TOP-2) = JB_MODE_ZPLUS  ) else
                 JB_combRes(1) when ( IVV(IVV_JB_MODE_TOP downto IVV_JB_MODE_TOP-2) = JB_MODE_ZMOINS ) else
                 JB_combRes(2) when ( IVV(IVV_JB_MODE_TOP downto IVV_JB_MODE_TOP-2) = JB_MODE_G      ) else
                 JB_combRes(3) when ( IVV(IVV_JB_MODE_TOP downto IVV_JB_MODE_TOP-2) = JB_MODE_LE     ) else
                 JB_bitRes;

   -- Compute value to add to PC depending on test result ; extending sign bit
   JB_valToAdd( 1 downto 0) <= "00";
   JB_valToAdd(15 downto 2) <= IVV(IVV_JB_RELATIV_TOP downto IVV_JB_RELATIV_TOP-13) when JB_globRes = '1' else
                               "00000000000000";
   jb0: for k in 31 downto 16 generate
     JB_valToAdd(k) <= JB_valToAdd(15);
   end generate;
   -- The address is comptuted into the ADDER : @see ADDER PART

   -- ==========================================================================================
   -- CPYB : bit copy/extraction between registers
   cpyb1 : cl_mux32x1 port map ( SEL  => IVV(IVV_CPYB_SRCB_TOP downto IVV_CPYB_SRCB_TOP-4),
                                 DIN  => DNN,
                                 DOUT => CPYB_bitSel
                               );
   CPYB_mode <= IVV(IVV_CPYB_MODE_TOP downto IVV_CPYB_MODE_TOP-1);
   cpyb2 : for i in 31 downto 0 generate
      CPYB_out(i) <= CPYB_bitSel when i = IVV(IVV_CPYB_DESTB_TOP downto IVV_CPYB_DESTB_TOP-4) else
                     DMM(i)      when CPYB_mode = CPYB_MODE_KEEP else
                     '0';
   end generate;
   CPYB_C <= CPYB_bitSel;

   -- ==========================================================================================
   -- ADDER
   add1 : cl_add32 port map (
                              Ci => ADD_Ci,
                              A  => ADD_A,
                              B  => ADD_B,
                              R  => ADD_out,
                              Co => ADD_Co
                            );
   ADDR_mode  <= IVV(IVV_ADDR_MODE_TOP downto IVV_ADDR_MODE_TOP-2);
   ADDD_value(15 downto 14) <= IVV(IVV_ADDD_VALUEH_TOP downto IVV_ADDD_VALUEH_TOP-1);
   ADDD_value(13 downto  0) <= IVV(IVV_ADDD_VALUEL_TOP downto IVV_ADDD_VALUEL_TOP-13);
   add2 : for i in 31 downto 16 generate
      ADDD_value(i) <= ADDD_value(15);
   end generate;
   process ( ALU_SELECT, ADDR_mode, FL, DNN, DMM, ADDD_value, JB_valToAdd )
   begin
         if ( ALU_SELECT(SELECT_ADDR) = '1' ) then
           ADD_A <=  DNN;
           case ADDR_mode is
             when ADDR_MODE_ADD  =>
                  ADD_Ci <= '0';
                  ADD_B  <= DMM;
             when ADDR_MODE_SUB  =>
                  ADD_Ci <= '1';
                  ADD_B  <= NOT(DMM);
             when ADDR_MODE_CMP  =>
                  ADD_Ci <= '1';
                  ADD_B  <= NOT(DMM);
             when ADDR_MODE_ADDC =>
                  ADD_Ci <= FL(FLAG_C32);
                  ADD_B  <= DMM;
             when ADDR_MODE_SUBC =>
                  ADD_Ci <= NOT(FL(FLAG_C32));
                  ADD_B  <= NOT(DMM);
             when others         =>
                  ADD_Ci <= 'X';
                  ADD_B  <= DMM;
           end case;
         elsif ( ALU_SELECT(SELECT_ADDD) = '1' ) then
                  ADD_Ci <= '0';
                  ADD_A  <= DNN;
                  ADD_B  <= ADDD_value;
         elsif ( ALU_SELECT(SELECT_JB) = '1' ) then
                  ADD_Ci <= '0';
                  ADD_A  <= JB_valToAdd;
                  ADD_B  <= DMM;
         else
                  ADD_Ci <= '0';
                  ADD_A  <= DNN;
                  ADD_B  <= DMM;
         end if;
   end process;


   -- ==========================================================================================
   -- MULTIPLIER
   MUL_mode <= '1' when IVV(IVV_MUL_MODE_TOP downto IVV_MUL_MODE_TOP-1) = MUL_MODE_SIGN else
               '0';
   mul1 : cl_full_mul32 port map (
                                   S => MUL_mode,
                                   A => DNN,
                                   B => DMM,
                                   R => MUL_out,
                                   C => MUL_C
                                  );
   -- ==========================================================================================
   -- DIVIDER
   DIV_sign <= '1' when IVV(IVV_DIV_MODE_TOP downto IVV_DIV_MODE_TOP-1) = DIV_MODE_SIGN else
               '0';
   div1 : cl_div32 port map (
                              S   => DIV_sign,
                              A   => DNN,
                              B   => DMM,
                              D   => DIV_res,
                              R   => DIV_rest,
                              RDY => DIV_rdy,
                              CLK => CLK,
                              STT => ALU_SELECT(SELECT_DIV)
                             );

   DIV_out(31 downto 16) <= DIV_res(31 downto  16) when  IVV(IVV_DIV_TYPE_TOP downto IVV_DIV_TYPE_TOP-1) = DIV_TYPE_RESULT else
                            DIV_rest(31 downto 16) when  IVV(IVV_DIV_TYPE_TOP downto IVV_DIV_TYPE_TOP-1) = DIV_TYPE_REST   else
                            DIV_rest(15 downto  0);
   DIV_out(15 downto  0) <= DIV_rest(15 downto  0) when  IVV(IVV_DIV_TYPE_TOP downto IVV_DIV_TYPE_TOP-1) = DIV_TYPE_REST   else
                            DIV_res(15 downto  0);

   DIV_cr <= not(DIV_rest(31)) when DIV_rest(31 downto 16) = X"FFFF" & '1' else
             not(DIV_rest(31)) when DIV_rest(31 downto 16) = X"0000" & '0' else
             DIV_rest(31);
   DIV_cd <= not(DIV_res(31)) when DIV_res(31 downto 16) = X"FFFF" & '1' else
             not(DIV_res(31)) when DIV_res(31 downto 16) = X"0000" & '0' else
             DIV_res(31);
   DIV_cg <= DIV_cr OR DIV_cd;

   DIV_c  <= DIV_cg when IVV(IVV_DIV_TYPE_TOP downto IVV_DIV_TYPE_TOP-1) = DIV_TYPE_NORMAL else
             DIV_out(31) when DIV_sign='1' else
             '0';


   -- ==========================================================================================
   -- SHIFT / ROTATION
   rot1 : cl_rot32 port map (
                              MODE => IVV(IVV_ROT_MODE_TOP downto IVV_ROT_MODE_TOP-2),
                              Ci   => FL(FLAG_C32),
                              DIR  => IVV(IVV_ROT_DIR_TOP),
                              STEP => IVV(IVV_ROT_STEP_TOP downto IVV_ROT_STEP_TOP-4),
                              I    => DNN,
                              O    => ROT_out,
                              Co   => ROT_C
                            );

   -- ==========================================================================================
   -- WRITE MEMORY Interface : CALL / STORE
   DATA_A <= DMM when  ALU_SELECT(SELECT_STORE) = '1' OR ALU_SELECT(SELECT_LOADm) = '1' else
             SP;                                                                                   -- PUSH, POP, CALL, CALLR

   DATA_O <= DNN;
   process( ALU_SELECT, DATA_I, IVV(IVV_STORE_MODE_TOP downto IVV_STORE_MODE_TOP-1) )
   begin
      if ( ALU_SELECT(SELECT_CALL) = '1' OR ALU_SELECT(SELECT_CALLR) = '1' ) then
          DATA_WE <= "1111";
      elsif ( ALU_SELECT(SELECT_STORE) = '1' OR ALU_SELECT(SELECT_PUSH) = '1' ) then
          if ( IVV(IVV_STORE_MODE_TOP downto IVV_STORE_MODE_TOP-1) = STORE_MODE_8 ) then
              DATA_WE <= "0001";
          elsif ( IVV(IVV_STORE_MODE_TOP downto IVV_STORE_MODE_TOP-1) = STORE_MODE_16 ) then
              DATA_WE <= "0011";
          else
              DATA_WE <= "1111";
          end if;
      else
          DATA_WE  <= "0000";
      end if;
   end process;


   -- ==========================================================================================
   -- Output routing

   DDD <= LOADd_OUT when ALU_ROUTE = ROUTE_LOADd else
          LOADx_OUT when ALU_ROUTE = ROUTE_LOADx else
          AND_out   when ALU_ROUTE = ROUTE_AND   else
          OR_out    when ALU_ROUTE = ROUTE_OR    else
          XOR_out   when ALU_ROUTE = ROUTE_XOR   else
          CPYB_out  when ALU_ROUTE = ROUTE_CPYB  else
          ADD_out   when ALU_ROUTE = ROUTE_ADD   else
          ADD_out   when ALU_ROUTE = ROUTE_JB    else
          MUL_out   when ALU_ROUTE = ROUTE_MUL   else
          DIV_out   when ALU_ROUTE = ROUTE_DIV   else
          ROT_out   when ALU_ROUTE = ROUTE_ROT   else
          DMM;                                           -- default : pass-throught


   C   <= CPYB_C    when ALU_ROUTE = ROUTE_CPYB  else
          ADD_Co    when ALU_ROUTE = ROUTE_ADD   else
          MUL_C     when ALU_ROUTE = ROUTE_MUL   else
          DIV_C     when ALU_ROUTE = ROUTE_DIV   else
          ROT_C     when ALU_ROUTE = ROUTE_ROT   else
          '0';

   ALU_RDY <= DIV_rdy when ALU_ROUTE = ROUTE_DIV else
              '1';


end arch_sc91_alu;
