-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    23:01 10/11/2005
-- Design Name:    cache_memory.vhd - Cache management
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  Xilinx independant
-- Tool versions:
-- Description:    Cache memory management to interface real SRAM interface
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library core_lib;
use core_lib.utils.ALL;


-- *****************************************************************
entity sc91_cache is
     port (
            C_ADDR   : IN  std_logic_vector( 31 downto  0 );      -- Code Address (32b aligned)
            C_DATA   : OUT std_logic_vector( 31 downto  0 );      -- Code data response
            C_RDY    : OUT std_logic;                             -- Code data ready (AH)

            D_ADDR   : IN  std_logic_vector( 31 downto  0 );      -- Data Address r/w start address
            D_DATAW  : IN  std_logic_vector( 31 downto  0 );      -- Data data to write into memory
            D_DATAR  : OUT std_logic_vector( 31 downto  0 );      -- Data data read from memory
            D_WE     : IN  std_logic_vector(  3 downto  0 );      -- Data request Write one bit per octet WE(0) -> ADDR ; WE(1) -> ADDR+1 ...
            D_RE     : IN  std_logic_vector(  3 downto  0 );      -- Data request Read  one bit per octet RE(0) -> ADDR ; RE(1) -> ADDR+1 ...
--            D_SIGN   : IN  std_logic;                             -- Requested number is signed.
            D_RDY    : OUT std_logic;                             -- Data operation done

            CLK      : IN  std_logic;                             -- Cache component clock
            RST      : IN  std_logic                              -- Cache component reset
          );
end entity sc91_cache;


architecture arch_sc91_cache of sc91_cache is
   -- Memory for code (temporary)
   component sc91_code is
       port (
            ADDRESS   : IN   std_logic_vector( 7 downto 0);
            DATA_OUT  : OUT  std_logic_vector(31 downto 0);
            IN_RDY    : OUT  std_logic;                        -- Instruction is ready - insert a bubble if no (AH)

            -- not use
            NU        : OUT std_logic                          -- not used, just that last line doesn't need ";"
         );
   end component;

   -- Memory interface logic for non aligned address.
   signal   i_WR0, i_WR1, i_WR2, i_WR3 : std_logic;

   signal   i_WE_w1    : std_logic_vector(3 downto 0);      -- Computed WE signal to match real byte word aligned
   signal   i_WE_w2    : std_logic_vector(3 downto 0);      -- Computed WE signal to match real byte word aligned

   signal   i_iword    : std_logic_vector(31 downto 0);     -- big endian word  word write
   signal   i_iword1   : std_logic_vector(31 downto 0);     -- first  word write
   signal   i_iword2   : std_logic_vector(31 downto 0);     -- second word write
   signal   i_word1    : std_logic_vector(31 downto 0);     -- first  word read
   signal   i_word2    : std_logic_vector(23 downto 0);     -- second word read
   signal   i_wordo32  : std_logic_vector(31 downto 0);     -- 1 word output word
   signal   i_wordo    : std_logic_vector(31 downto 0);     -- final signed 1 output word
   signal   i_ld_word2 : std_logic;                         -- request to store word into word2 register

   signal   i_A1A0     : std_logic_vector(1 downto 0);         -- Data Address b0 b1 mirror.
   signal   i_aligned  : std_logic;                            -- Simple Memory access : word is aligned, only one read/write is need
   signal   i_access   : std_logic_vector(3 downto 1);         -- Read or Write operation for each bit
   signal   i_rd, i_wr : std_logic;                            -- Read and Write operation

   signal   t_sign_8   : std_logic_vector( 7 downto 0);        -- temporary sign propagation
   signal   t_sign_16  : std_logic_vector(15 downto 0);        -- temporary sign propagation
   signal   t_sign_24  : std_logic_vector(23 downto 0);        -- temporary sign propagation

   signal   i_DADDR    : std_logic_vector(31 downto 0);        -- internal memory address
   signal   i_DADDR1   : std_logic_vector(31 downto 0);        -- internal memory address +1
   signal   i_SELADDR  : std_logic;                            -- select address to READ/WRITE 0=i_DADDR 1=i_DADDR1


   signal   i_DDATAW   : std_logic_vector(31 downto 0);        -- internal data to write

   type STATE_SRAM is ( q_wait, q_rd_1, q_wr_1 );
   attribute ENUM_ENCODING: STRING;
   attribute ENUM_ENCODING of STATE_SRAM: type is "00 01 10";
   signal i_STATE, n_STATE : STATE_SRAM;

begin
   -- ########################################################################
   -- CODE MEMORY INTERFACE
   -- ########################################################################
   i_code : sc91_code port map (
            ADDRESS  => C_ADDR(7 downto 0),
            DATA_OUT => C_DATA,
            IN_RDY   => C_RDY
            );

   -- ########################################################################
   -- DATA MEMORY INTERFACE
   -- ########################################################################
   i_A1A0      <= D_ADDR(1 downto 0);
   i_access(1) <= D_WE(1) OR D_RE(1);
--   i_access(2) <= D_WE(2) OR D_RE(2);
   i_access(3) <= D_WE(3) OR D_RE(3);
   i_aligned   <= '0' when i_A1A0 = "11" AND i_access(1) = '1' else
                  '0' when i_A1A0 = "10" AND i_access(3) = '1' else
                  '0' when i_A1A0 = "01" AND i_access(3) = '1' else
                  '1';

--   i_aligned   <= '1' when i_A1A0 = "00" else                           -- return 1 when the data requested only impact one
--                  '1' when i_A1A0 = "01" AND i_access(3) = '0' else     -- 32 bit word.
--                  '1' when i_A1A0 = "10" AND i_access(2) = '0' else
--                  '1' when i_A1A0 = "11" AND i_access(1) = '0' else
--                  '0';
   i_wr        <= D_WE(3) OR D_WE(2) OR D_WE(1) OR D_WE(0);
   i_rd        <= D_RE(3) OR D_RE(2) OR D_RE(1) OR D_RE(0);

   process (CLK, RST)
   begin
      if ( RST = '1' ) then
         i_STATE <= q_wait;
      elsif ( CLK = '1' and CLK'event ) then
         i_STATE <= n_STATE;
      end if;
   end process;

   process ( i_STATE, i_aligned, i_wr, i_rd )
   begin
     case i_STATE is
      when q_wait  => if ( i_rd = '1' and i_aligned = '0' ) then
                        n_STATE    <= q_rd_1;
                        D_RDY      <= '0';
                        i_SELADDR  <= '1';    -- Read Word + 1 first
                        i_LD_WORD2 <= '1';    -- Store Word + 1
                      elsif ( i_wr = '1' and i_aligned = '0' ) then
                        n_STATE    <= q_wr_1;
                        D_RDY      <= '0';
                        i_SELADDR  <= '1';    -- Write Word + 1 first
                        i_LD_WORD2 <= '0';    -- No way
                      else
                        n_STATE    <= q_wait;
                        D_RDY      <= '1';
                        i_SELADDR  <= '0';    -- Default only 1 word read/write
                        i_LD_WORD2 <= '0';    -- Second word not required
                      end if;

      when q_rd_1 =>    n_STATE    <= q_wait;
                        D_RDY      <= '1';
                        i_SELADDR  <= '0';    -- Read Word
                        i_LD_WORD2 <= '0';    -- Don't update

      when q_wr_1 =>    n_STATE    <= q_wait;
                        D_RDY      <= '1';
                        i_SELADDR  <= '0';    -- Read Word
                        i_LD_WORD2 <= '0';    -- Don't update

      when others =>    n_STATE    <= q_wait;
                        D_RDY      <= '0';
                        i_SELADDR  <= '0';
                        i_LD_WORD2 <= '0';

     end case;
   end process;


   -- =================== INPUT INTERFACE ======================================
   -- Compute a Big Endian word depending en word to write len
   process ( D_WE, D_DATAW )
   begin
      if ( D_WE(3) = '1' ) then
         i_iword( 7 downto  0) <= D_DATAW(31 downto 24);
         i_iword(15 downto  8) <= D_DATAW(23 downto 16);
         i_iword(23 downto 16) <= D_DATAW(15 downto  8);
         i_iword(31 downto 24) <= D_DATAW( 7 downto  0);
      elsif ( D_WE(1) = '1' ) then
         i_iword( 7 downto  0) <= D_DATAW(15 downto  8);
         i_iword(15 downto  8) <= D_DATAW( 7 downto  0);
         i_iword(31 downto 16) <= ( others => 'X' );
      else -- ( D_WE(0) = '1' ) then
         i_iword( 7 downto  0) <= D_DATAW( 7 downto  0);
         i_iword(31 downto  8) <= ( others => 'X' );
      end if;
  end process;
  -- Split this new 32 value into two word depending on adress
  -- compute new i_WE_wn
  process( i_A1A0, D_DATAW, D_WE, i_iword )
  begin
      case i_A1A0 is
         when  "00"  =>  i_iword1(31 downto  0) <= i_iword(31 downto  0);
                         i_iword2(31 downto  0) <= (others => 'X');
                         i_WE_w1(0) <= D_WE(0); i_WE_w1(1) <= D_WE(1); i_WE_w1(2) <= D_WE(3); i_WE_w1(3) <= D_WE(3);
                         i_WE_w2(0) <= '0';     i_WE_w2(1) <= '0';     i_WE_w2(2) <= '0';     i_WE_w2(3) <= '0';

         when  "01"  =>  i_iword1( 7 downto  0) <= (others => 'X');
                         i_iword1(31 downto  8) <= i_iword(23 downto  0);
                         i_iword2( 7 downto  0) <= i_iword(31 downto 24);
                         i_iword2(31 downto  8) <= (others => 'X');
                         i_WE_w1(0) <= '0';     i_WE_w1(1) <= D_WE(0); i_WE_w1(2) <= D_WE(1); i_WE_w1(3) <= D_WE(3);
                         i_WE_w2(0) <= D_WE(3); i_WE_w2(1) <= '0';     i_WE_w2(2) <= '0';     i_WE_w2(3) <= '0';

         when  "10"  =>  i_iword1(15 downto  0) <= (others => 'X');
                         i_iword1(31 downto 16) <= i_iword(15 downto  0);
                         i_iword2(15 downto  0) <= i_iword(31 downto 16);
                         i_iword2(31 downto 16) <= (others => 'X');
                         i_WE_w1(0) <= '0';     i_WE_w1(1) <= '0';     i_WE_w1(2) <= D_WE(0); i_WE_w1(3) <= D_WE(1);
                         i_WE_w2(0) <= D_WE(3); i_WE_w2(1) <= D_WE(3); i_WE_w2(2) <= '0';     i_WE_w2(3) <= '0';

         when others =>  i_iword1(31 downto 24) <= i_iword( 7 downto  0);
                         i_iword1(23 downto  0) <= (others => 'X');
                         i_iword2(31 downto 24) <= (others => 'X');
                         i_iword2(23 downto  0) <= i_iword(31 downto  8);
                         i_WE_w1(0) <= '0';     i_WE_w1(1) <= '0';     i_WE_w1(2) <= '0';     i_WE_w1(3) <= D_WE(0);
                         i_WE_w2(0) <= D_WE(1); i_WE_w2(1) <= D_WE(3); i_WE_w2(2) <= D_WE(3); i_WE_w2(3) <= '0';
      end case;
   end process;


--   -- Compute real WR enable signal depending on memory start address
--   -- then signe it.
--   process ( i_A1A0, D_DATAW, D_WE )
--   begin
--      case i_A1A0 is
--         when  "00"  =>  i_WE_w1(0) <= D_WE(0); i_WE_w1(1) <= D_WE(1); i_WE_w1(2) <= D_WE(2); i_WE_w1(3) <= D_WE(3);
--                         i_WE_w2(0) <= '0';     i_WE_w2(1) <= '0';     i_WE_w2(2) <= '0';     i_WE_w2(3) <= '0';
--                         i_iword1(31 downto  0) <= D_DATAW(31 downto  0);
--                         i_iword2(31 downto  0) <= X"00000000";
--
--         when  "01"  =>  i_WE_w1(0) <= '0';     i_WE_w1(1) <= D_WE(0); i_WE_w1(2) <= D_WE(1); i_WE_w1(3) <= D_WE(2);
--                         i_WE_w2(0) <= D_WE(3); i_WE_w2(1) <= '0';     i_WE_w2(2) <= '0';     i_WE_w2(3) <= '0';
--                         i_iword1(31 downto  8) <= D_DATAW(23 downto  0); i_iword1( 7 downto  0) <= X"00";
--                         i_iword2(31 downto  8) <= X"000000";             i_iword2( 7 downto  0) <= D_DATAW(31 downto 24);
--
--         when  "10"  =>  i_WE_w1(0) <= '0';     i_WE_w1(1) <= '0';     i_WE_w1(2) <= D_WE(0); i_WE_w1(3) <= D_WE(1);
--                         i_WE_w2(0) <= D_WE(2); i_WE_w2(1) <= D_WE(3); i_WE_w2(2) <= '0';     i_WE_w2(3) <= '0';
--                         i_iword1(31 downto 16) <= D_DATAW(15 downto  0); i_iword1(15 downto  0) <= X"0000";
--                         i_iword2(31 downto 16) <= X"0000";               i_iword2(15 downto  0) <= D_DATAW(31 downto 16);
--
--         when others =>  i_WE_w1(0) <= '0';     i_WE_w1(1) <= '0';     i_WE_w1(2) <= '0';     i_WE_w1(3) <= D_WE(0);
--                         i_WE_w2(0) <= D_WE(1); i_WE_w2(1) <= D_WE(2); i_WE_w2(2) <= D_WE(0); i_WE_w2(3) <= '0';
--                         i_iword1(31 downto 24) <= D_DATAW( 7 downto  0); i_iword1(23 downto  0) <= X"000000";
--                         i_iword2(31 downto 24) <= X"00";                 i_iword2(23 downto  0) <= D_DATAW(31 downto  8);
--
--      end case;
--   end process;

   -- Route memory address with D_ADDR or D_ADDR+4
   addr_add : cl_add4 port map( B => D_ADDR, R => i_DADDR1 );
   i_DADDR <= D_ADDR when i_SELADDR = '0' else i_DADDR1;

   -- Generate 32bit register used to store word1
   i_reg24b : cl_reg generic map ( WIDTH => 24 )
                     port map ( D => i_word1(23 downto 0), Q => i_word2, CLK => CLK, LOAD => '1', RESET => RST); -- LOAD = i_ld_word2

   -- Generate byte selection signal for memory write command
   i_WR0    <= i_WE_w1(0) when i_SELADDR = '0' else i_WE_w2(0);
   i_WR1    <= i_WE_w1(1) when i_SELADDR = '0' else i_WE_w2(1);
   i_WR2    <= i_WE_w1(2) when i_SELADDR = '0' else i_WE_w2(2);
   i_WR3    <= i_WE_w1(3) when i_SELADDR = '0' else i_WE_w2(3);

   -- Route corresponding word to write
   i_DDATAW <= i_iword1 when i_SELADDR = '0' else i_iword2;


   -- =================== CONNECTING TO AN EXTERNAL MEMORY BLOCK =================
   i_data : cl_ram16x32 port map (
            I     => i_DDATAW,
            O     => i_word1,
            WR0   => i_WR0,
            WR1   => i_WR1,
            WR2   => i_WR2,
            WR3   => i_WR3,
            WCLK  => CLK,
            A     => i_DADDR(5 downto 2) );


   -- =================== OUTPUT INTERFACE ======================================
   -- Compute output level to Map two word on a sigle one as required
--   process ( i_A1A0, D_RE, i_word1, i_word2 )
--   begin
--      case i_A1A0 is
--         when  "00"  =>  i_wordo32(31 downto  0) <= i_word1(31 downto  0);
--
--         when  "01"  =>  i_wordo32(31 downto 24) <= i_word2( 7 downto  0);
--                         i_wordo32(23 downto  0) <= i_word1(31 downto  8);
--
--         when  "10"  =>  i_wordo32(31 downto 16) <= i_word2(15 downto  0);
--                         i_wordo32(15 downto  0) <= i_word1(31 downto 16);
--
--         when others =>  i_wordo32(31 downto  8) <= i_word2(23 downto  0);
--                         i_wordo32( 7 downto  0) <= i_word1(31 downto 24);
--
--      end case;
--   end process;

   process ( i_A1A0, i_word1, i_word2 )
   begin
      case i_A1A0 is
         when  "00"  =>  i_wordo32(31 downto  0) <= i_word1(31 downto  0);

         when  "01"  =>  i_wordo32(31 downto  8) <= i_word1(31 downto  8);
                         i_wordo32( 7 downto  0) <= i_word2( 7 downto  0);

         when  "10"  =>  i_wordo32(31 downto 16) <= i_word1(31 downto 16);
                         i_wordo32(15 downto  0) <= i_word2(15 downto  0);

         when others =>  i_wordo32(31 downto 24) <= i_word1(31 downto 24);
                         i_wordo32(23 downto  0) <= i_word2(23 downto  0);
      end case;
   end process;

   -- Compute an standard output word depending on Big-Endian word read
   process ( D_RE, i_wordo32 )
   begin
      if ( D_RE(3) = '1' ) then
         i_wordo( 7 downto  0) <= i_wordo32(31 downto 24);
         i_wordo(15 downto  8) <= i_wordo32(23 downto 16);
         i_wordo(23 downto 16) <= i_wordo32(15 downto  8);
         i_wordo(31 downto 24) <= i_wordo32( 7 downto  0);
      elsif ( D_RE(1) = '1' ) then
         i_wordo( 7 downto  0) <= i_wordo32(15 downto  8);
         i_wordo(15 downto  8) <= i_wordo32( 7 downto  0);
         i_wordo(23 downto 16) <= i_wordo32(15 downto  8);     --No way : just to simplify routing
         i_wordo(31 downto 24) <= i_wordo32( 7 downto  0);     --No way : just to simplify routing
      else  --( D_WE(0) = '1' ) then
         i_wordo( 7 downto  0) <= i_wordo32( 7 downto  0);
         i_wordo(15 downto  0) <= (others => 'X');             --No way : compilo, do as best
         i_wordo(23 downto 16) <= i_wordo32(15 downto  8);     --No way : just to simplify routing
         i_wordo(31 downto 24) <= i_wordo32( 7 downto  0);     --No way : just to simplify routing
      end if;
  end process;

  D_DATAR <= i_wordo;

--   D_DATAR <= i_wordo32;

   -- Compute a signed value for memory, mask unwanted bit
--   process ( D_RE, i_wordo32 ) -- i_A1A0, D_RE, i_wordo32, t_sign_8, t_sign_16, t_sign_24 )
--   begin
--      if( D_RE(3) = '1' ) then
--         i_wordo(31 downto  0) <= i_wordo32(31 downto 0);
--      elsif ( D_RE(2) = '1' ) then
--         i_wordo(23 downto  0) <= i_wordo32(23 downto 0);
--         i_wordo(31 downto 24) <= (others => i_wordo32(23));   --t_sign_8 ( 7 downto 0);
--      elsif ( D_RE(1) = '1' ) then
--         i_wordo(15 downto  0) <= i_wordo32(15 downto 0);
--         i_wordo(31 downto 16) <= (others => i_wordo32(15));   --t_sign_16(15 downto 0);
--      else
--         i_wordo( 7 downto  0) <= i_wordo32( 7 downto 0);
--         i_wordo(31 downto  8) <= (others => i_wordo32(7));    -- t_sign_24(23 downto 0);
--      end if;
--   end process;
--   D_DATAR <= i_wordo;


   -- ***********************************************
   -- Sign propagation ....
--   gs0: for g in 7 downto 0 generate
--      t_sign_8(g) <= i_wordo32(23) when D_SIGN='1' else '0';
--   end generate;
--   gs1: for g in 15 downto 0 generate
--      t_sign_16(g) <= i_wordo32(15) when D_SIGN='1' else '0';
--   end generate;
--   gs2: for g in 23 downto 0 generate
--      t_sign_24(g) <= i_wordo32(7) when D_SIGN='1' else '0';
--   end generate;


end arch_sc91_cache;

