-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    19:47:00 09/23/05
-- Design Name:    code_memory.vhdl - SC91 alu
--
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    simple-cpu code memory for tests
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library core_lib;
use core_lib.utils.ALL;


-- *****************************************************************************
-- ALU ENTITY DESCRIPTION
-- *****************************************************************************
entity sc91_code is
   port (
        ADDRESS   : IN   std_logic_vector( 7 downto 0);
        DATA_OUT  : OUT  std_logic_vector(31 downto 0);
        IN_RDY    : OUT  std_logic;                        -- Instruction is ready - insert a bubble if no (AH)

        -- not use
        NU        : OUT std_logic                          -- not used, just that last line doesn't need ";"
   );
end sc91_code;

architecture arch_sc91_code of sc91_code is
begin

   IN_RDY <= '1';
   i_romCode : cl_rom64x32 generic map ( INIT =>
       ( "11111100001000100001100100001000",                       -- 11111100 00100 0100001100100001 00 0 - LOAD R04,0x4321,low           0
         "11111100001001000011101100101010",                       -- 11111100 00100 1000011101100101 01 0 - LOAD R04,0x8765,high          4
         "11111100001011010010110100101000",                       -- 11111100 00101 1010010110100101 00 0 - LOAD R05,0xA5A5,low           8
         "11111100001011010010110100101010",                       -- 11111100 00101 1010010110100101 01 0 - LOAD R05,0xA5A5,high          C
         "11110001001000010000101000000000",                       -- 11110001 00100 00100 00101 000000000 - AND R04,R04,R05              10
         "00000000000000000000000000000111",                       -- 00 000000000000000000000000000111    - JMP 1C                       14
         "11110011001000010000100000000000",                       -- 11110011 00100 00100 00100 000000000 - XOR R04,R04,R04              18
         "11110010001000010000101000000000",                       -- 11110010 00100 00100 00101 000000000 - OR  R04,R04,R05              1C
         "11110011001000010000101000000000",                       -- 11110011 00100 00100 00101 000000000 - XOR R04,R04,R05              20
         "11101000000000010000000000000000",                       -- 11101000 00000 00100 000000000000    - JMP @R04                     24
         X"00000000",                             --                                                                               10
         X"00000000",                             --                                                                               11
         X"00000000",                             --                                                                               12
         X"00000000",                             --                                                                               13
         X"00000000",                             --                                                                               14
         X"00000000",                             --                                                                               15

         X"70000007",                             --                                                                               16
         X"70000007",                             --                                                                               17
         X"70000007",                             --                                                                               18
         X"70000007",                             --                                                                               19
         X"70000007",                             --                                                                               20
         X"70000007",                             --                                                                               21
         X"70000007",                             --                                                                               22
         X"70000007",                             --                                                                               23
         X"70000007",                             --                                                                               24
         X"70000007",                             --                                                                               25
         X"70000007",                             --                                                                               26
         X"70000007",                             --                                                                               27
         X"70000007",                             --                                                                               28
         X"70000007",                             --                                                                               29
         X"70000007",                             --                                                                               30
         X"70000007",                             --                                                                               31

         X"60000006",                             --                                                                               32
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --
         X"60000006",                             --                                                                               47

         X"50000005",                             --                                                                               48
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005",                             --
         X"50000005")                             --                                                                               63

--         X"40000004",                             --                                                                               64
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--        X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --
--         X"40000004",                             --                                                                               79

--         X"30000003",                             --                                                                               80
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --
--         X"30000003",                             --                                                                               95

--         X"20000002",                             --                                                                               96
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --
--         X"20000002",                             --                                                                              111

--         X"10000001",                             --                                                                              112
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001",                             --
--         X"10000001" )                             --                                                                              127

                            )
                            port map ( O => DATA_OUT, A => ADDRESS(7 downto 2) );



end arch_sc91_code;
