-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    22:30:00 09/29/05
-- Design Name:    instruction_set.vhd - insctruction set constants
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    describes constant to represent OPCODE et linked stuff
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

package INSTRUCTION_SET is


   -- ==========================================================================================
   -- FLAG BITS
   constant FLAG_Z       : INTEGER := 0;
   constant FLAG_S8      : INTEGER := 1;
   constant FLAG_S16     : INTEGER := 2;
   constant FLAG_S32     : INTEGER := 3;
   constant FLAG_O8      : INTEGER := 4;
   constant FLAG_O16     : INTEGER := 5;
   constant FLAG_O32     : INTEGER := 6;
   constant FLAG_C8      : INTEGER := 7;
   constant FLAG_C16     : INTEGER := 8;
   constant FLAG_C32     : INTEGER := 9;
   constant FLAG_P       : INTEGER := 10;
   constant FLAG_0       : INTEGER := 11;

   -- ==========================================================================================
   -- OPCODE CONSTANTS
   constant OPCODE_LOADD : std_logic_vector(7 downto 0) := "11111100";       -- LOADd
   constant OPCODE_LOADR : std_logic_vector(7 downto 0) := "11111110";       -- LOADr
   constant OPCODE_LOADM : std_logic_vector(7 downto 0) := "11111101";       -- LOADm
   constant OPCODE_STORE : std_logic_vector(7 downto 0) := "11101100";       -- STORE
   constant OPCODE_NOP   : std_logic_vector(7 downto 0) := "11111111";       -- NOP
   constant OPCODE_AND   : std_logic_vector(7 downto 0) := "11110001";       -- AND
   constant OPCODE_OR    : std_logic_vector(7 downto 0) := "11110010";       -- OR
   constant OPCODE_XOR   : std_logic_vector(7 downto 0) := "11110011";       -- XOR
   constant OPCODE_JMPR  : std_logic_vector(7 downto 0) := "11101000";       -- JMPr
   constant OPCODE_CPYB  : std_logic_vector(7 downto 0) := "11100000";       -- CPYB
   constant OPCODE_ADDR  : std_logic_vector(7 downto 0) := "10000000";       -- ADDR
   constant OPCODE_MUL   : std_logic_vector(7 downto 0) := "11111000";       -- MUL
   constant OPCODE_DIV   : std_logic_vector(7 downto 0) := "11111001";       -- DIV
   constant OPCODE_ROT   : std_logic_vector(7 downto 0) := "11110100";       -- ROT
   constant OPCODE_PUSH  : std_logic_vector(7 downto 0) := "11100100";       -- PUSH
   constant OPCODE_POP   : std_logic_vector(7 downto 0) := "11100101";       -- POP
   constant OPCODE_CALLR : std_logic_vector(7 downto 0) := "11101001";       -- CALLr
   --constant OPCODE_ADDD  : std_logic_vector(7 downto 0) := "101000xx";     -- ADDd


   -- ==========================================================================================
   -- ALU ROUTING OPCODE RECODING
   type t_recoding is array (22 downto 0) of std_logic;
   constant SELECT_LOADD   : INTEGER :=  0;           -- LOADd
   constant SELECT_AND     : INTEGER :=  1;           -- AND
   constant SELECT_OR      : INTEGER :=  2;           -- OR
   constant SELECT_XOR     : INTEGER :=  3;           -- XOR
   constant SELECT_LOADr   : INTEGER :=  4;           -- LOADr
   constant SELECT_LOADm   : INTEGER :=  5;           -- LOADm
   constant SELECT_POP     : INTEGER :=  6;           -- POP
   constant SELECT_JB      : INTEGER :=  7;           -- JB
   constant SELECT_CPYB    : INTEGER :=  8;           -- CPYB
   constant SELECT_ADDR    : INTEGER :=  9;           -- ADDr
   constant SELECT_ADDD    : INTEGER := 10;           -- ADDd
   constant SELECT_MUL     : INTEGER := 11;           -- MUL
   constant SELECT_DIV     : INTEGER := 12;           -- MUL
   constant SELECT_ROT     : INTEGER := 13;           -- ROT
   constant SELECT_CALL    : INTEGER := 14;           -- CALL
   constant SELECT_CALLR   : INTEGER := 15;           -- CALL
   constant SELECT_PUSH    : INTEGER := 16;           -- PUSH
   constant SELECT_STORE   : INTEGER := 17;           -- STORE - same route as Push
   constant SELECT_JMPR    : INTEGER := 18;           -- JMPR - DNN Pass Throught
   constant SELECT_JMP     : INTEGER := 19;           -- JMP  - DDD no way
   constant SELECT_NOP     : INTEGER := 20;           -- NOP  - DDD no way
   constant SELECT_DEFAULT : INTEGER := 21;           -- DEFAULT SELECTION

   type t_route is array (3 downto 0) of std_logic;
   constant ROUTE_LOADd    : t_route := "0001";       -- LOADd
   constant ROUTE_LOADx    : t_route := "0010";       -- LOADr, LOADm, POP
   constant ROUTE_AND      : t_route := "0011";       -- AND
   constant ROUTE_OR       : t_route := "0100";       -- OR
   constant ROUTE_XOR      : t_route := "0101";       -- XOR
   constant ROUTE_CPYB     : t_route := "0110";       -- CPYB
   constant ROUTE_JB       : t_route := "0111";       -- JB
   constant ROUTE_ADD      : t_route := "1000";       -- ADDr, ADDd
   constant ROUTE_MUL      : t_route := "1001";       -- MUL
   constant ROUTE_DIV      : t_route := "1010";       -- MUL
   constant ROUTE_ROT      : t_route := "1011";       -- ROT
   constant ROUTE_DEFAULT  : t_route := "0000";       -- JMPr, NOP, JMP, CALL, STORE, PUSH, CALLr

   -- ==========================================================================================
   -- INSTRUCTION LOADd
   constant LOADd_MODE_LOW      : std_logic_vector ( 1 downto 0) := "00";
   constant LOADd_MODE_HIGH     : std_logic_vector ( 1 downto 0) := "01";
   constant LOADd_MODE_SIGNED   : std_logic_vector ( 1 downto 0) := "10";
   constant LOADd_MODE_UNSIGNED : std_logic_vector ( 1 downto 0) := "11";
   constant IVV_LOADd_VALUE_TOP : INTEGER := 18;      -- Higher bit for LOADd 16b Value
   constant IVV_LOADd_MODE_TOP  : INTEGER := 2;       -- Higher bit for LOADd 2b mode

   -- INSTRUCTION LOADr / LOADm / POP
   constant LOADx_MODE_8U       : std_logic_vector ( 2 downto 0) := "000";
   constant LOADx_MODE_8S       : std_logic_vector ( 2 downto 0) := "100";
   constant LOADx_MODE_16U      : std_logic_vector ( 2 downto 0) := "001";
   constant LOADx_MODE_16S      : std_logic_vector ( 2 downto 0) := "101";
   constant LOADx_MODE_32       : std_logic_vector ( 2 downto 0) := "010";
   constant IVV_LOADx_MODE_TOP  : INTEGER := 8;       -- Higher bit for LOADr 3b mode

   -- INSTRUCTION STORE
   constant STORE_MODE_8        : std_logic_vector ( 1 downto 0) := "00";
   constant STORE_MODE_16       : std_logic_vector ( 1 downto 0) := "01";
   constant STORE_MODE_32       : std_logic_vector ( 1 downto 0) := "10";
   constant IVV_STORE_MODE_TOP  : INTEGER := 8;       -- Higher bit for LOADr 2b mode

   -- INSTRUCTION JB
   constant IVV_JB_MODE_TOP     : INTEGER := 27;      -- Higher bit for JB 3b mode
   constant IVV_JB_BIT_TOP      : INTEGER := 23;      -- Higher bit for bit number to test
   constant IVV_JB_V_TOP        : INTEGER := 24;      -- Higher bit for bit value
   constant IVV_JB_RELATIV_TOP  : INTEGER := 13;      -- Higher bit for JB relative 14 bit address
   constant JB_MODE_BIT         : std_logic_vector ( 2 downto 0) := "000";
   constant JB_MODE_ZPLUS       : std_logic_vector ( 2 downto 0) := "010";
   constant JB_MODE_ZMOINS      : std_logic_vector ( 2 downto 0) := "011";
   constant JB_MODE_G           : std_logic_vector ( 2 downto 0) := "110";
   constant JB_MODE_LE          : std_logic_vector ( 2 downto 0) := "111";

   -- INSTRUCTION CPYB
   constant IVV_CPYB_SRCB_TOP   : INTEGER := 4;      -- Higher bit for CPYB 5bits source bit mask
   constant IVV_CPYB_DESTB_TOP  : INTEGER := 13;     -- Higher bit for CPYB 5bits destination bit mask
   constant IVV_CPYB_MODE_TOP   : INTEGER := 8;      -- Higher bit for CPYB 2bits mode
   constant CPYB_MODE_CLEAR     : std_logic_vector ( 1 downto 0) := "00";
   constant CPYB_MODE_KEEP      : std_logic_vector ( 1 downto 0) := "10";

   -- INSTRUCTION ADDr
   constant IVV_ADDR_MODE_TOP   : INTEGER := 8;      -- Higher bit for ADDr 3bits mode
   constant ADDR_MODE_ADD       : std_logic_vector ( 2 downto 0) := "000";
   constant ADDR_MODE_SUB       : std_logic_vector ( 2 downto 0) := "100";
   constant ADDR_MODE_CMP       : std_logic_vector ( 2 downto 0) := "110";
   constant ADDR_MODE_ADDC      : std_logic_vector ( 2 downto 0) := "001";
   constant ADDR_MODE_SUBC      : std_logic_vector ( 2 downto 0) := "101";

   -- INSTRUCTION ADDd
   constant IVV_ADDD_VALUEH_TOP : INTEGER := 25;      -- 2 highest bit for direct value
   constant IVV_ADDD_VALUEL_TOP : INTEGER := 13;      -- 14 lowest bit for direct value

   -- INSTRUCTION MUL
   constant IVV_MUL_MODE_TOP    : INTEGER := 8;       -- Higher bit for MUL 2 bits mode
   constant MUL_MODE_SIGN       : std_logic_vector ( 1 downto 0) := "00";     -- signed multiplier
   constant MUL_MODE_UNSIGN     : std_logic_vector ( 1 downto 0) := "10";     -- unsigned multiplier

   -- INSTRUCTION DIV
   constant IVV_DIV_MODE_TOP    : INTEGER := 8;       -- Higher bit for DIV 2 bits mode
   constant DIV_MODE_SIGN       : std_logic_vector ( 1 downto 0) := "00";     -- signed divider
   constant DIV_MODE_UNSIGN     : std_logic_vector ( 1 downto 0) := "10";     -- unsigned divider
   constant IVV_DIV_TYPE_TOP    : INTEGER := 6;       -- Higher bit for MUL 2 bits mode
   constant DIV_TYPE_NORMAL     : std_logic_vector ( 1 downto 0) := "11";     -- Rest into 16b high, result into 16b low
   constant DIV_TYPE_REST       : std_logic_vector ( 1 downto 0) := "01";     -- Destination gets 32b rest value
   constant DIV_TYPE_RESULT     : std_logic_vector ( 1 downto 0) := "10";     -- Destination gets 32b result value

   -- INSTRUCTION ROT
   constant IVV_ROT_MODE_TOP    : INTEGER := 7;       -- Higher bit for MUL 3 bits mode
   constant IVV_ROT_DIR_TOP     : INTEGER := 8;       -- Higher bit for MUL 1 bits direction
   constant IVV_ROT_STEP_TOP    : INTEGER := 13;      -- Higher bit for MUL 5 bits step




end INSTRUCTION_SET;




package body INSTRUCTION_SET is

end INSTRUCTION_SET;


