-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    20:39:36 08/11/05
-- Design Name:    registers-bank.vhdl - SC91 register bank as defined on
--                                       http://www.simple-cpu.com/architecture-bloc-registres-fr.php
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    simple-cpu register bank implementation
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library core_lib;
use core_lib.utils.ALL;

library work;
use work.instruction_set.ALL;

-- *****************************************************************************
-- REGISTERS-BANK ENTITY DESCRIPTION
-- *****************************************************************************
entity registers_bank is
   port (
        RST    : IN  std_logic;                         -- reset signal (Active High AH)
        CLK    : IN  std_logic;                         -- system clock

        -- Register selection
        RNN    : IN  std_logic_vector(4 downto 0);      -- first source operand
        RMM    : IN  std_logic_vector(4 downto 0);      -- second source operand
        RDD    : IN  std_logic_vector(4 downto 0);      -- destination operand
        LDD    : IN  std_logic;                         -- load RDD register with DDD value (AH)


        -- Data buses
        DNN    : OUT  std_logic_vector(31 downto 0);    -- first source operand data bus
        DMM    : OUT  std_logic_vector(31 downto 0);    -- second source operand data bus
        DDD    : IN   std_logic_vector(31 downto 0);    -- destination operand data bus

        -- Specific allways connected PC regsiter
        PC     : OUT  std_logic_vector(31 downto 0);    -- Program counter register output
        PCDIR  : IN   std_logic_vector(31 downto 0);    -- Program counter register direct input
        PC_R00 : IN   std_logic_vector(1 downto 0);     -- bit 1 control PC output as 1 => PCDIR 0 => R00
                                                        -- bit 0 control PC input  as 1 => Ddd   0 => PC + 4
        PC_LOAD: IN   std_logic;                        -- load PC with its input value (AH)

        -- Specific flag signals
        C      : IN  std_logic;                         -- carry signal from ALU
        FL     : OUT std_logic_vector(15 downto 0);     -- computed flags for ALU and Instruction
        LFL    : IN  std_logic;                         -- update Flags with current result (AH)

        -- Specific Stack pointer signals
        SP     : OUT std_logic_vector(31 downto 0);     -- SP Register value for PUSH/POP/CALL instructions
        SPP    : IN  std_logic;                         -- store in SP (R01) the current value + 4   (AH)
        SPM    : IN  std_logic;                         -- put on R01 output current value - 4       (AH)
                                                        -- store in SP (R01) the current value - 4

        R04    : OUT std_logic_vector(31 downto 0);     -- debugging facility
        -- not use
        NU     : OUT std_logic                           -- not used, just that last line doesn't need ";"

   );
end registers_bank;

architecture arch_registers_bank of registers_bank is

   constant NUM_OF_REG : INTEGER := 32;                     -- num of registers, must be greater than 3.
   constant NUM_OF_SEL : INTEGER := 5;                      -- num of registers, must be greater than 3.
   constant REG_WIDTH  : INTEGER := 32;                     -- num of bit for each registers

   type TypeRxx is array (NUM_OF_REG-1 downto 0) of  std_logic_vector(31 downto 0);

   signal Rxx : TypeRxx;                                    -- Registers output buses
   signal Lxx : std_logic_vector(NUM_OF_REG-1 downto 0);   -- Register individual Load command
   signal Lxt : std_logic_vector(NUM_OF_REG-1 downto 0);   -- temporary signal

   signal R00_input  : std_logic_vector(31 downto 0);      -- R00 input bus after Mux selecting Ddd/R00+4
   signal R00_output : std_logic_vector(31 downto 0);      -- R00 input bus after Mux selecting Ddd/R00+4
   signal R00_plu4   : std_logic_vector(31 downto 0);      -- R00 plus 4
   signal R00_ddd    : std_logic_vector(31 downto 0);      -- R00 load from ddd (% 4)
   signal R00_dir    : std_logic_vector(31 downto 0);      -- R00 direct load (% 4)
   signal R01_input  : std_logic_vector(31 downto 0);      -- R01 input bus after Mux selecting Ddd/R01+4/R01-4
   signal R01_output : std_logic_vector(31 downto 0);      -- R01 direct output
   signal R01_min4   : std_logic_vector(31 downto 0);      -- R01 minus 4
   signal R01_plu4   : std_logic_vector(31 downto 0);      -- R01 plus 4
   signal R02_input  : std_logic_vector(31 downto 0);      -- R02 input bus after Mux selecting Ddd/Flags
   signal Flag_value : std_logic_vector(31 downto 0);      -- Flag value bus used for computing each bit

   signal Rxx_mod    : std_logic_vector((REG_WIDTH*NUM_OF_REG)-1 downto 0 );
                                                            -- other way to see the same ... explicit cast
begin
   -- ==========================================================================================
   -- Generic Registers
   -- ==========================================================================================
   -- ------------------------------------------------------------------------------------------
   -- Generic registers LOAD - to load a register it has to be selected by its number with RDD
   --                          and LDD command has to be set.
   -- ------------------------------------------------------------------------------------------
   i_demux_L : cl_demux32 port map ( SEL => RDD, DIN => LDD, DOUT => Lxt);
   g_lxx : for k in 3 to (NUM_OF_REG-1) generate
               Lxx(k) <= Lxt(k);
   end generate;

   g_rxx : for k in 3 to 15 generate                                        -- (NUM_OF_REG-1) generate
     i_reg32b_k : cl_reg generic map ( WIDTH => REG_WIDTH )
                         port map ( D => DDD, Q => Rxx(k), CLK => CLK, LOAD => Lxx(k), RESET => RST);
   end generate;


   -- ==========================================================================================
   -- Specific Register R00 - Program counter
   -- ==========================================================================================
   Lxx(0) <= PC_LOAD;
   i_r00_add4 : cl_add4 port map ( B => R00_output, R => R00_plu4);
   R00_ddd(REG_WIDTH-1 downto 2) <= DDD(REG_WIDTH-1 downto 2);
   R00_ddd(1 downto 0) <= "00";
   R00_input <= R00_ddd when ( PC_R00(0) = '1' ) else
                R00_plu4;

   i_r00 : cl_reg generic map ( WIDTH => REG_WIDTH-2 )
                  port map ( D => R00_input(REG_WIDTH-1 downto 2),
                             Q => Rxx(0)(REG_WIDTH-1 downto 2),
                             CLK => CLK, LOAD => Lxx(0), RESET => RST);
   Rxx(0)(1 downto 0) <= "00";
   R00_dir(REG_WIDTH-1 downto 2) <= PCDIR(REG_WIDTH-1 downto 2);
   R00_dir(1 downto 0) <= "00";
   R00_output <= R00_dir when ( PC_R00(1) = '1' ) else
                 Rxx(0);
   PC <= R00_output;

   -- ==========================================================================================
   -- Specific Register R01 - Stack pointer
   --                         Input will be DDD when LDD is set and address match, else it
   --                         will be R01+4 if SPP is set or R01-4 if SPM is set. In other cases
   --                         value is unchanged
   -- ==========================================================================================
   i_r01 : cl_reg generic map ( WIDTH => REG_WIDTH )
                  port map ( D => R01_input, Q => R01_output, CLK => CLK, LOAD => Lxx(1), RESET => RST);
   i_r01_sub4 : cl_sub4 port map ( B => R01_output, R => R01_min4);
   i_r01_add4 : cl_add4 port map ( B => R01_output, R => R01_plu4);

   R01_input <= R01_plu4 when (SPP = '1') else
                R01_min4 when (SPM = '1') else
                DDD;
   Lxx(1)    <= ( SPM AND LDD ) OR ( SPP AND LDD ) OR Lxt(1);
   Rxx(1)    <= R01_output when ( SPM = '0' ) else R01_min4;

   -- ==========================================================================================
   -- Specific Register R02 - Flags register
   --                         Input will be DDD when LDD is set and address match, else it
   --                         will be update from Flag computing when FLF is UP. LDD has the priority
   --                         F(11) = 0 ; F(10) = Parity eq 1 when number of 1 is even
   --                         F(09/08/07) = 32b/16b/8b carry, F(06/05/04) = 32b/16b/8b overflow
   --                         F(03/02/01) = 32b/16b/8b sign, F(00) = eq 1 when DDD value is 0
   -- ==========================================================================================
   R02_input <= Flag_value when (LFL = '1') else DDD;
   Lxx(2)    <= LFL OR Lxt(2);
   i_r02 : cl_reg generic map ( WIDTH => REG_WIDTH )
                  port map ( D => R02_input, Q => Rxx(2), CLK => CLK, LOAD => Lxx(2), RESET => RST);
   Flag_value(31) <= Rxx(2)(31);
   Flag_value(30 downto 12) <= Rxx(2)(30 downto 12);
   Flag_value(FLAG_0)   <= '0';
   Flag_value(FLAG_P)   <= NOT(    DDD(0)  XOR DDD(1)  XOR DDD(2)  XOR DDD(3)  XOR DDD(4)  XOR DDD(5)  XOR DDD(6)  XOR DDD(7)
                               XOR DDD(8)  XOR DDD(9)  XOR DDD(10) XOR DDD(11) XOR DDD(12) XOR DDD(13) XOR DDD(14) XOR DDD(15)
                               XOR DDD(16) XOR DDD(17) XOR DDD(18) XOR DDD(19) XOR DDD(20) XOR DDD(21) XOR DDD(22) XOR DDD(23)
                               XOR DDD(24) XOR DDD(25) XOR DDD(26) XOR DDD(27) XOR DDD(28) XOR DDD(29) XOR DDD(30) XOR DDD(31)
                              );
   Flag_value(FLAG_C32) <= C;
   Flag_value(FLAG_C16) <= DDD(16);
   Flag_value(FLAG_C8)  <= DDD(9);
   Flag_value(FLAG_O32) <= DDD(31) XOR C;
   Flag_value(FLAG_O16) <= DDD(15) XOR DDD(16);
   Flag_value(FLAG_O8)  <= DDD(7) XOR DDD(8);
   Flag_value(FLAG_S32) <= DDD(31);
   Flag_value(FLAG_S16) <= DDD(15);
   Flag_value(FLAG_S8)  <= DDD(7);
   Flag_value(FLAG_Z)   <= NOT(   DDD(0)  OR DDD(1)  OR DDD(2)  OR DDD(3)  OR DDD(4)  OR DDD(5)  OR DDD(6)  OR DDD(7)
                               OR DDD(8)  OR DDD(9)  OR DDD(10) OR DDD(11) OR DDD(12) OR DDD(13) OR DDD(14) OR DDD(15)
                               OR DDD(16) OR DDD(17) OR DDD(18) OR DDD(19) OR DDD(20) OR DDD(21) OR DDD(22) OR DDD(23)
                               OR DDD(24) OR DDD(25) OR DDD(26) OR DDD(27) OR DDD(28) OR DDD(29) OR DDD(30) OR DDD(31)
                              );

   FL(15 downto 0) <= Rxx(2)(15 downto 0);

   -- ==========================================================================================
   -- Output interfaces to Dnn and Dmm operand buses
   -- ==========================================================================================
   g_rww_mod : for k in 0 to (NUM_OF_REG-1) generate
     Rxx_mod( (REG_WIDTH*(k+1)-1) downto (REG_WIDTH*k) ) <= Rxx(k)(31 downto 0);
   end generate;

   i_mux_N : cl_mux32 port map ( SEL => RNN, DIN =>Rxx_mod , DOUT => DNN);
   i_mux_M : cl_mux32 port map ( SEL => RMM, DIN =>Rxx_mod , DOUT => DMM);

   R04 <= Rxx(4);
   SP  <= Rxx(1);


end arch_registers_bank;
