-- ********************************************************************************
-- Copyright  Paul Pinault, Disk91 2005-06.
--
-- This program is free software/hardware; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- See : http://www.gnu.org/licenses/gpl.txt
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--
-- Paul Pinault, Disk91 nor Simple-CPU terms can't be used in the way to promote
-- nor legitimate a derived product without the Author authorization.
--
-- =================================================================================
-- Using This software made your own application to become a free-software too
-- considered under GPL. In this case you will have to publish your modificated
-- VHDL code at least on a public website.
-- Perhaps, it's possible to contact the author at ( dpl AT disk91.com ) to obtain
-- the way to have a license spcific distribution of it with or without any fee
-- depending on your project.
-- =================================================================================
--
-- Ce programme est un logiciel / matriel libre, vous pouvez le redistribuer et/ ou
-- le modifier selon les termes de la GNU  General Public License comme publi par
-- la Free Software Fundation ; tablie en version 2 ou suprieure.
--
-- La license en Franais peut tre consultable sur le site suivant :
-- http://fsffrance.org/gpl/gpl-fr.fr.html
--
-- CE PROGRAMME EST FOURNI  TEL QU'EN L'TAT , SANS AUCUNE GARANTIE DE
-- QUELQUE NATURE, IMPLICITE OU EXPLICITE, QUANT  SON UTILISATION
-- COMMERCIALE, PROFESSIONNELLE, LGALE OU NON, OU AUTRE. L'AUTEUR NE PEUT
-- EN AUCUN CAS TRE TENU POUR RESPONSABLE DE QUELQUE DOMMAGE OU PRJUDICE
-- DIRECT, INDIRECT, SECONDAIRE OU ACCESSOIRE DCOULANT DE L'UTILISATION DE
-- TOUT OU PARTIE DE CE PROGRAMME.
--
-- Les noms Paul Pinault, Disk91 ou Simple-CPU ne saurait tre utiliss dans le but
-- de promouvoir ou de lgitimer un produit driv de ce programme sans accord pralable
-- de l'auteur.
-- ********************************************************************************
--------------------------------------------------------------------------------
-- Company:        www.simple-cpu.com
-- Engineer:       Paul Pinault
--
-- Create Date:    20:39:36 08/11/05
-- Design Name:    registers.vhd - registers basic library
-- Module Name:    sc91-A - Behavioral
-- Project Name:   sc91-A
-- Target Device:  independant
-- Tool versions:
-- Description:    standard elements to build registers, may be latter map on
--                 target optimized one.
--
-- Dependencies:   none
--
-- Revision:       1.0
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


library core_lib;
use core_lib.utils.ALL;

library work;
use work.instruction_set.ALL;

entity sc91_A is
   port ( CLK         : IN  std_logic;                         -- 50 MHZ clock on T9
          RESET       : IN  std_logic;

          MANUAL_CLK  : IN  std_logic;                         -- control clock manually
          seg7high    : IN  std_logic;                         -- print high bit of 7 seg value
          caddr       : OUT std_logic_vector(4 downto 0);
          slowclk     : OUT std_logic;
          manclk      : OUT std_logic;
          seg7        : OUT std_logic_vector(6 downto 0);
          seg7ctrl    : OUT std_logic_vector(3 downto 0)
        );
end sc91_A;


architecture arch_sc91_A of sc91_A is

   -- #################################################################################################################
   -- INCLUDE EXTERNAL COMPONENT
   -- #################################################################################################################
      component registers_bank is
         port (
            RST    : IN  std_logic;                         -- reset signal (Active High AH)
            CLK    : IN  std_logic;                         -- system clock

            -- Register selection
            RNN    : IN  std_logic_vector(4 downto 0);      -- first source operand
            RMM    : IN  std_logic_vector(4 downto 0);      -- second source operand
            RDD    : IN  std_logic_vector(4 downto 0);      -- destination operand
            LDD    : IN  std_logic;                         -- load RDD register with DDD value (AH)


            -- Data buses
            DNN    : OUT  std_logic_vector(31 downto 0);    -- first source operand data bus
            DMM    : OUT  std_logic_vector(31 downto 0);    -- second source operand data bus
            DDD    : IN   std_logic_vector(31 downto 0);    -- destination operand data bus

            -- Specific allways connected PC regsiter
            PC     : OUT  std_logic_vector(31 downto 0);    -- Program counter register output
            PCDIR  : IN   std_logic_vector(31 downto 0);    -- Program counter register direct input
            PC_R00 : IN   std_logic_vector(1 downto 0);     -- control PC output as R00 (00) Ddd (01) PCDIR (10)
            PC_LOAD: IN   std_logic;                        -- load PC with its input value (AH)

            -- Specific flag signals
            C      : IN  std_logic;                         -- carry signal from ALU
            FL     : OUT std_logic_vector(15 downto 0);     -- computed flags for ALU and Instruction
            LFL    : IN  std_logic;                         -- update Flags with current result (AH)

            -- Specific Stack pointer signals
            SP     : OUT std_logic_vector(31 downto 0);     -- SP Register value for PUSH/POP/CALL instructions
            SPP    : IN  std_logic;                         -- store in SP (R01) the current value + 4   (AH)
            SPM    : IN  std_logic;                         -- put on R01 output current value - 4       (AH)
                                                            -- store in SP (R01) the current value - 4

            R04    : OUT std_logic_vector(31 downto 0);     -- debugging facility
            -- not use
            NU     : OUT std_logic                          -- not used, just that last line doesn't need ";"

         );
      end component;

      component sc91_sequenceur is
         port (

            RST    : IN   std_logic;                         -- reset signal (Active High AH)
            CLK    : IN   std_logic;                         -- system clock

            -- Instructions
            IVV    : OUT  std_logic_vector(31 downto 0);     -- instruction bit for ALU
            INSTR  : IN   std_logic_vector(31 downto 0);     -- Intruction read from memory
            IN_RDY : IN   std_logic;                         -- Instruction is ready - insert a bubble if no (AH)

            -- Specific Data memory insterface
            DA_RDY : IN   std_logic;                         -- Data is ready - stay on the same instruction if no (AH)

            -- Specific control signals
            ALU_SELECT: OUT   t_recoding;                    -- Indicate to ALU what output to choose (instruction recoding)
            ALU_ROUTE : OUT   t_route;                       -- choose ALU output depending on instruction

            -- Register selection
            RNN    : OUT  std_logic_vector(4 downto 0);      -- first source operand
            RMM    : OUT  std_logic_vector(4 downto 0);      -- second source operand
            RDD    : OUT  std_logic_vector(4 downto 0);      -- destination operand
            LDD    : OUT  std_logic;                         -- load RDD register with DDD value (AH)

            -- Control PC
            PCDIR  : OUT  std_logic_vector(31 downto 0);    -- Program counter register direct input
            PC_R00 : OUT  std_logic_vector( 1 downto 0);    -- control PC output as R00 (00) Ddd (01) PCDIR (10)
            PC_LOAD: OUT  std_logic;                        -- load PC with its input value (AH)

            -- Specific flag signals
            FL     : IN  std_logic_vector(15 downto 0);     -- computed flags for ALU and Instruction
            LFL    : OUT std_logic;                         -- update Flags with current result (AH)

            -- Specific Stack pointer signals
            SPP    : OUT std_logic;                         -- store in SP (R01) the current value + 4   (AH)
            SPM    : OUT std_logic;                         -- put on R01 output current value - 4       (AH)

            -- not use
            NU        : OUT std_logic                       -- not used, just that last line doesn't need ";"

         );
      end component;

      component sc91_alu is
         port (

            -- Instruction data input - instruction bit 27 downto 0 could contain interesting information
            IVV       : IN   std_logic_vector(31 downto 0);    -- instruction bit for mode and direct data

            -- Data buses
            DNN       : IN   std_logic_vector(31 downto 0);    -- first source operand data bus
            DMM       : IN   std_logic_vector(31 downto 0);    -- second source operand data bus
            DDD       : OUT  std_logic_vector(31 downto 0);    -- destination operand data bus

            -- Specific flag signals
            C         : OUT  std_logic;                         -- carry signal from ALU
            FL        : IN   std_logic_vector(15 downto 0);     -- computed flags for ALU and Instruction

            -- Stack signal
            SP        : IN  std_logic_vector(31 downto 0);      -- SP Register value for PUSH/POP/CALL instructions

            -- Specific control signals
            ALU_SELECT: IN   t_recoding;                        -- Indicate to ALU what output to choose (instruction recoding)
            ALU_ROUTE : IN   t_route;                           -- choose ALU output depending on instruction
            CLK       : IN   std_logic;                         -- clock signal for division
            ALU_RDY   : OUT  std_logic;                         -- requested result is ready

            -- Memory interface
            DATA_A    : OUT  std_logic_vector(31 downto 0);    -- Data memory interface : address
            DATA_O    : OUT  std_logic_vector(31 downto 0);    -- Data memory interface : data to write into memory
            DATA_WE   : OUT  std_logic_vector( 3 downto 0);    -- Indicate Write Len (1, 2 or 4 bytes = 0001 0011 1111)
            DATA_RE   : OUT  std_logic_vector( 3 downto 0);    -- Indicate Read  Len (1, 2 or 4 bytes = 0001 0011 1111)
            DATA_I    : IN   std_logic_vector(31 downto 0);    -- Data memory interface : data to read into memory

            -- not use
            NU        : OUT std_logic                          -- not used, just that last line doesn't need ";"

         );
      end component;

      -- Memory interface
      component sc91_cache is
         port (
                  C_ADDR   : IN  std_logic_vector( 31 downto  0 );      -- Code Address (32b aligned)
                  C_DATA   : OUT std_logic_vector( 31 downto  0 );      -- Code data response
                  C_RDY    : OUT std_logic;                             -- Code data ready (AH)

                  D_ADDR   : IN  std_logic_vector( 31 downto  0 );      -- Data Address r/w start address
                  D_DATAW  : IN  std_logic_vector( 31 downto  0 );      -- Data data to write into memory
                  D_DATAR  : OUT std_logic_vector( 31 downto  0 );      -- Data data read from memory
                  D_WE     : IN  std_logic_vector(  3 downto  0 );      -- Data request Write one bit per octet WE(0) -> ADDR ; WE(1) -> ADDR+1 ...
                  D_RE     : IN  std_logic_vector(  3 downto  0 );      -- Data request Read  one bit per octet RE(0) -> ADDR ; RE(1) -> ADDR+1 ...
                  D_RDY    : OUT std_logic;                             -- Data operation done

                  CLK      : IN  std_logic;                             -- Cache component clock
                  RST      : IN  std_logic                              -- Cache component reset
               );
      end component;


   -- #################################################################################################################
   -- INTERNAL SIGNALS
   -- #################################################################################################################
   signal slow_clock : std_logic;
   signal fast_clock : std_logic;
   signal clock_divider : std_logic_vector(24 downto 0);
   signal rand_value : std_logic_vector(31 downto 0);

   signal i_DNN      : std_logic_vector(31 downto 0);
   signal i_DMM      : std_logic_vector(31 downto 0);
   signal i_DDD      : std_logic_vector(31 downto 0);
   signal i_RNN      : std_logic_vector( 4 downto 0);
   signal i_RMM      : std_logic_vector( 4 downto 0);
   signal i_RDD      : std_logic_vector( 4 downto 0);

   signal i_CTRL     : std_logic_vector(15 downto 0);
   signal i_FL       : std_logic_vector(15 downto 0);
   signal i_IVV      : std_logic_vector(31 downto 0);
   signal i_PCDIR    : std_logic_vector(31 downto 0);
   signal i_PC       : std_logic_vector(31 downto 0);
   signal i_PCR00    : std_logic_vector( 1 downto 0);
   signal i_INSTR    : std_logic_vector(31 downto 0);

   signal i_PCLOAD   : std_logic;
   signal i_INRDY    : std_logic;
   signal i_DARDY    : std_logic;
   signal i_ALURDY   : std_logic;
   signal i_ALLRDY   : std_logic;
   signal i_LDD      : std_logic;
   signal i_LFL      : std_logic;
   signal i_SPM      : std_logic;
   signal i_SPP      : std_logic;
   signal i_SP       : std_logic_vector(31 downto 0);
   signal i_C        : std_logic;

   signal i_DATA_A   : std_logic_vector(31 downto 0);
   signal i_DATA_O   : std_logic_vector(31 downto 0);
   signal i_DATA_I   : std_logic_vector(31 downto 0);
   signal i_DATA_WE  : std_logic_vector( 3 downto 0);
   signal i_DATA_RE  : std_logic_vector( 3 downto 0);

   signal i_clk      : std_logic;
   signal i_manual_clk : std_logic;
   signal i_segaig   : std_logic_vector(15 downto 0);
   signal i_R04      : std_logic_vector(31 downto 0);       -- debugging output R04

   signal i_select   : t_recoding;
   signal i_route    : t_route;

begin

   process (CLK, clock_divider)
   begin
      if ( CLK = '1' and CLK'event ) then
       clock_divider <= clock_divider + 1;
      end if;
   end process;
   slow_clock <= clock_divider(24);       -- about 1Hz
   fast_clock <= clock_divider(15);


   process (slow_clock,MANUAL_CLK)
   begin
      if ( slow_clock = '1' and slow_clock'event ) then
        if ( MANUAL_CLK = '1' ) then
          i_manual_clk <= NOT(i_manual_clk);
        end if;
      end if;
   end process;


   --i_R04 <= i_INSTR;
   i_clk <= i_manual_clk;
   i_segaig <= i_R04(15 downto 0) when seg7high='1' else i_R04(31 downto 16);
   i_7seg_X : cl_7segX port map ( D => i_segaig,
                                  CLK => fast_clock, CTRL => seg7ctrl, Q => seg7 );
   slowclk <= slow_clock;
   manclk  <= i_manual_clk;
   caddr   <= i_PC(6 downto 2);


   i_memory : sc91_cache port map (
            C_ADDR   => i_PC,                                     -- Code memory interface
            C_DATA   => i_INSTR,
            C_RDY    => i_INRDY,

            D_ADDR   => i_DATA_A,                                 -- Data memory interface
            D_DATAW  => i_DATA_O,
            D_DATAR  => i_DATA_I,
            D_WE     => i_DATA_WE,
            D_RE     => i_DATA_RE,
            D_RDY    => i_DARDY,

            CLK      => i_CLK,
            RST      => RESET );


   i_ALLRDY <= i_DARDY AND i_ALURDY;
   i_seq : sc91_sequenceur port map (
            RST      => RESET,
            CLK      => i_clk,

            INSTR    => i_INSTR,
            IN_RDY   => i_INRDY,
            DA_RDY   => i_ALLRDY,

            ALU_SELECT => i_select,
            ALU_ROUTE  => i_route,

            IVV      => i_IVV,
            RNN      => i_RNN,
            RMM      => i_RMM,
            RDD      => i_RDD,
            LDD      => i_LDD,
            PCDIR    => i_PCDIR,
            PC_R00   => i_PCR00,
            PC_LOAD  => i_PCLOAD,
            FL       => i_FL,
            LFL      => i_LFL,
            SPP      => i_SPP,
            SPM      => i_SPM );

    i_alu : sc91_alu port map (
            IVV      => i_IVV,
            DNN      => i_DNN,
            DMM      => i_DMM,
            DDD      => i_DDD,
            C        => i_C,
            FL       => i_FL,
            SP       => i_SP,

            ALU_SELECT => i_select,
            ALU_ROUTE  => i_route,
            CLK        => i_clk,
            ALU_RDY    => i_ALURDY,

            DATA_A   => i_DATA_A,
            DATA_I   => i_DATA_I,
            DATA_WE  => i_DATA_WE,
            DATA_RE  => i_DATA_RE,
            DATA_O   => i_DATA_O );

   i_reg_bank : registers_bank port map(
            RST      => RESET,
            CLK      => i_clk,

            RNN      => i_RNN,
            RMM      => i_RMM,
            RDD      => i_RDD,
            LDD      => i_LDD,

            DNN      => i_DNN,
            DMM      => i_DMM,
            DDD      => i_DDD,

            PC       => i_PC,
            PCDIR    => i_PCDIR,
            PC_R00   => i_PCR00,
            PC_LOAD  => i_PCLOAD,

            C        => i_C,
            FL       => i_FL,
            LFL      => i_LFL,

            SP       => i_SP,
            SPP      => i_SPP,
            SPM      => i_SPM

            ,R04     => i_R04
            );


end arch_sc91_A;
